#!/usr/bin/perl

# test perl script  
#
#  Loads all HTML files in the current directory (one by one) and makes 
#   sure that all tags and attributes are capitalized
#
#  Wade Lutgen October 1999

@files = <*.html>;

# Loop over all files found
foreach $file (@files)
{
  print STDOUT "Converting $file\n";
  open(FILE, $file);
 
  $original_file = "";

  # grab all lines from within the file and put them in original_file string
  while (<FILE>) 
  {
    $original_file .= $_;
  }
  $modified_file = $original_file;

  # pull anything out of the original file that is between '<' and '>' and 
  # place it in $substring
  while($original_file =~ /(<.+?>)/gs)
  {
      $substring = $1;
      $flag = 0;

      # Capitalize HTML opening tag  (first word after '<' in substring)
      #  checks for beginning of string, an opening brace '<',  0 or more 
      #  whitespace characters, a word boundary,
      #  0 or more word characters, 1 or more lowercase letters,
      #  0 or more word characters, and finally a word break.
      if ($substring =~ /(^<(\s*?)(\b)(\w*?)([a-z]+?)(\w*?)(\b))/ )
      {
        $temp = $1;
        $temp =~ tr/a-z/A-Z/;
        $modified_file =~ s/$1/$temp/;
      } 

      # Capitalize HTML tag attributes  (word(s) before '=' in substring) 
      # There may be more than one, so loop
      #  checks for a word boundary,
      #  0 or more word characters, 1 or more lowercase characters
      #  0 or more word characters, a word boundary, 
      #  0 or more spaces, and then an equals sign
      while ($substring =~ /((\b)(\w*?)([a-z]+?)(\w*?)(\b)(\s*?)=)/g ) 
      {
        $temp = $1;
        $temp =~ tr/a-z/A-Z/;
        $modified_file =~ s/$1/$temp/;
      }

      # Capitalize HTML closing tags  (word after '</' in substring)
      #  checks for the beginning of string, an opening brace '<',  0 or more 
      #  spaces, a /, 1 or more lowercase letters, 0 or more spaces, and a 
      #  closing >
      if ($substring =~ /(^<(\s*?)(\/)(\b)(\w*?)([a-z]+?)(\w*?)(\b))/ )
      {
        $temp = $1;
        $temp =~ tr/a-z/A-Z/;
        $modified_file =~ s/$1/$temp/;
      }
  }

  # close readable file 
  close(FILE);

  # open same file for writing
  open(FILE, ">$file");
  print FILE $modified_file;
  close(FILE);
}
