/*
 * This example is from the book "Java Enterprise in a Nutshell".
 * Copyright (c) 1999 by O'Reilly & Associates.  
 * You may distribute this source code for non-commercial purposes only.
 * You may study, modify, and use this example for any purpose, as long as
 * this notice is retained.  Note that this example is provided "as is",
 * WITHOUT WARRANTY of any kind either expressed or implied.
 */

package entity.containerManaged;

import NoSuchPersonException;
import javax.ejb.*;
import java.rmi.RemoteException;

/**
 * The "home" interface for the ProfileBean.  This interface provides
 * methods used to create beans on the server.  The container provider is
 * responsible for implementing this interface, most likely using
 * auto-generated Java classes derived from the interface bytecodes.
 */

public interface ProfileHome extends EJBHome {
  /**
   * Create a new (nameless) profile
   */
  public Profile create() throws CreateException, RemoteException;

  /**
   * Create a profile for a named person.  Throws an exception if the person's
   * profile can't be found.
   */
  public Profile create(String name)
    throws RemoteException, NoSuchPersonException;

  /**
   * Lookup a Profile by name (the primary key)
   */
  public Profile findByPrimaryKey(ProfilePK key)
    throws RemoteException, FinderException;

  /**
   * Lookup a Profile by the value of a particular entry in the profile.
   */
//   public Enumeration findByEntryValue(String key, String value)
//     throws RemoteException, FinderException;
}
