package ccm.ct.util;

//a number of utility methods

import javax.servlet.ServletContext;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

public class CTUtilities {


    int MaxFileSize = 1000000;

    public String skipChars(String inpString, String charsToSkip) {
	// deletes chars from inpString if they are present in charsToSkip

	int len = inpString.length();
	char ch;
	StringBuffer buff = new StringBuffer(len);
	for (int i = 0; i < len; i++) {
	    ch = inpString.charAt(i);
	    if(charsToSkip.indexOf(ch) == -1) {
                buff.append((char)ch);
	    }
	}
	return buff.toString();
    }


    public String userDir (HttpServletRequest request,
                                 ServletContext application) {
	// returns string with path to user directory
	String userName = request.getRemoteUser();
	String contextDir = application.getRealPath("/");
	String path = contextDir + "WEB-INF" 
	    + File.separator + "Descriptors" + File.separator
	    + "users" + File.separator + userName;
	return path;
    }

    public String templateDir (ServletContext application) {
	// returns the string to the top of template directory 
        String contextDir = application.getRealPath("/");
        String path = contextDir + "WEB-INF" +
            File.separator + "Templates";
        return path;
    }

    public String dataDir (ServletContext application) {
	// returns the string to the top of the Data directory 
        String contextDir = application.getRealPath("/");
        String path = contextDir + "WEB-INF"
            + File.separator + "Data";
        return path;
    }

    public String replacePiece(String oldPiece, String newPiece, String line) {
	// replaces all occurances of substring oldPiece with newPiece in line
	int oldPieceLen = oldPiece.length();
	int newPieceLen = newPiece.length();
	int lineLen = line.length();
	int i = 0;
	while ((i = line.indexOf(oldPiece, i)) > 0 ) {
	    line = line.substring(0, i) + newPiece +
		line.substring(i+oldPieceLen);
	    i += newPieceLen;
	}
	return line;
    }

    public String replaceTags (Hashtable namesAndValues, String line) {
	// replaces tags @@**key**@@  with values using elements of 
	// namesAndValues hashtable
	Enumeration e = namesAndValues.keys();
	while (e.hasMoreElements()) {
	    String key = (String)e.nextElement(); 
	    String value = (String)namesAndValues.get(key);
	    key = "@@**" + key + "**@@";
	    line = replacePiece(key, value, line);
	}
	return line;
    }

    public void processTemplateFile(String sourceTemplateFile, 
				    String targetResultingFile,
				    Hashtable namesAndValues) {
	// reads file sourceTemplateFile line by line, and replaces all
	// the tags specified by namesAndValues, and writes the lines to
	// file targetResultingFile
	//This is a debug ==============
	System.err.println("procTempFile in CTUtils: Source=" + sourceTemplateFile);
	System.err.println("targetResultingFile=" + targetResultingFile);
	Enumeration e = namesAndValues.keys();
	while (e.hasMoreElements()) {
	    String key = (String)e.nextElement(); 
	    String value = (String)namesAndValues.get(key);
	    System.err.println("namesAndValues["+key+"]="+value);
	}
	//This is a debug ==============
 
	try {
	    BufferedReader in = 
		new BufferedReader(new FileReader(sourceTemplateFile));
	    BufferedWriter out = 
		new BufferedWriter(new FileWriter(targetResultingFile));
	    String line;
	    while((line = in.readLine()) != null) {
		line = replaceTags(namesAndValues, line);
		out.write(line);
		out.newLine();
	    }
	    in.close();
	    out.close();
	}
	catch (Exception ex) {
	    System.err.println(ex.getMessage());
	    ex.printStackTrace();
	}
    }


    public String FileToString(HttpServletRequest request,
			       ServletContext application,
			       String fileName) {

	// returns the string which contains the content of the file.
	String userTop = userDir(request, application);
        int len;

        File file = new File(fileName);
        if(! file.isFile()) {
            System.out.println("StringAndFile: " + fileName +
                               "not a file");
            return null;
        }


        if(! file.exists()) {
            System.out.println("FileToString: " + fileName + " not there?");
            return null;
        }

        if(! file.canRead()) {
            System.out.println("FileToString: " + fileName + " not there?");
            return null;
        }

        if((len = (int)file.length()) > MaxFileSize) {
            System.out.println("FileToString: " + fileName + " is " +
                               len + "byte long");
            return null;
        }
        if(!fileName.startsWith(userTop)) {
            System.out.println("FileToString: The " + fileName +
                               " does not start with " + userTop);
        }

        try {
            FileInputStream fs = new FileInputStream(file);
            byte[] buff = new byte [1000000];
            int count = fs.read(buff);
            String content = new String (buff);
            return content;
        }
        catch (IOException e) {
            System.err.println("FileToString.java Could not read");
        }
	return null;
    }
    //========================================================================
    public boolean StringToFile(HttpServletRequest request,
			        ServletContext application,
			        String fileName,
                                String goToFile) {
	// takes the string (goToFile) and writes it a file (fileName).
	// makes sure that the current user has permission to do this. 

	// writes string to a file.
	String userTop = userDir(request, application);
        int len;

        File file = new File(fileName);

	//if(! file.canWrite()) {
	//  System.out.println("StringToFile: "+fileName+" cannot write!");
	//  return false;
	//  }

        if(!fileName.startsWith(userTop)) {
            System.out.println("StringToFile: The " + fileName +
                               " does not start with " + userTop);
        }

        try {
            FileOutputStream fs = new FileOutputStream(file);
	    fs.write(goToFile.getBytes());
	    fs.close();
	    return true;
        }
        catch (IOException e) {
            System.err.println("StringToFile.java Could not write");
        }
	return false;
    }

    //========================================================================
    public String execCommandLocally(String command) {
	// Executes a UNIX "command" on a local host.
	// This command can do anything, incuding halting the machine or
	// deleting the /etc/passwd file. With this command:
	// "There is no try". Either you are 100% sure you know what you do,
	// tested it from command line, and checked spelling 10 times, or you
	// do not do it. If you ask it ( cd /; rm -rf * ) it will do it.
	// But you will not ask it, will you?      
	// Return string with command, status, stdout, stderr

	Runtime rt = Runtime.getRuntime();
	System.err.println("execCommandOnBackend -- command: "+ command);

	String commandOutput = "COMMAND: " + command+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(command);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //========================================================================
    public String execCommandOnBackend(String host,
				       String user,
				       String command) {
	// Executes a UNIX "command" on a remote "host" as "user" using ssh.
	// E.g.: clut.execCommandOnBackend("origin.osc.edu","pse",
	//                   "( cd pse; ls -lR * )");  
	// Before this can be done, the user/machine which runs the
	// webserver/tomcat has to be authorized to be able to ssh without
	// password to the account "user@host". While there are several ways
	// of accomplishing that, the one we used was to use ssh-keygen
	// as a user which runs the webserver/tomcat  (the user has to
	// have home directory even if he does not have a shell like
	// nobody), then append the copy of the identity.pub so created
	// to the file ${HOME}/.ssh/auhorized_keys on "user@host" account.
	// Return string with command, status, stdout, stderr

	Runtime rt = Runtime.getRuntime();
	String sshCommand = "ssh " + "-l " + user + " " + host + " " + command;
	System.err.println("execCommandOnBackend -- command: "+ sshCommand);

	String commandOutput = "COMMAND: " + sshCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(sshCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //======================================================
    public String copyFileFromBackend(String scpOptions,
				      String user,
				      String host,
				      String remoteFile,
				      String localFile) {
	// This method uses scp to copy files from remote host
	// to the local file. This is very dangerous command (e.g.,
	// can overwrite important files, so make sure that local file
	// does not point at some important directory or file.
	// The arguments are used in the scp command:  
	//ctu.copyFileFromBackend("origin.osc.edu", "pse", "p", 
	//  "bin/*", "/tmp")
        // produces: scp -p pse@origin.osc.edu:bin/* /tmp
	// Note, in normal circumstances the 'pse@origin.osc.edu:bin/*'
	// would be taken in single or double quotes to avaid shell
	// metachar processing. In our case, the shell interpretor
	// is not used.
	// Return string with command, status, stdout, stderr


	Runtime rt = Runtime.getRuntime();
        if(scpOptions.length() > 0) {
	    String scpOptionsTrim = scpOptions.trim();
	    if(! scpOptionsTrim.startsWith("-")) {
		scpOptions = "-" + scpOptionsTrim;
	    } 
	}

	String scpCommand = "scp " + scpOptions + " " + user.trim()
	    + "@" + host.trim() +  ":" + remoteFile.trim() + " " +
	    localFile;
	System.err.println("copyFileFromBackend  -- command: "+ scpCommand);

	String commandOutput = "COMMAND: " + scpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(scpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }
    //======================================================
    public String copyFileToBackend(String scpOptions,
				    String localFile,
				    String user,
				    String host,
				    String remoteFile) {
	// This method uses scp to copy files from webserver
	// to remote hosts. This is very dangerous command (e.g.,
	// can overwrite important files, so make sure that remote file
	// does not point at some important directory or file.
	// The arguments are used in the scp command:  
	//ctu.copyFileToBackend("p", "bin/*", "pse", "origin.osc.edu", 
	//  "/tmp", )
        // produces: scp -p pse@origin.osc.edu:bin/* /tmp
	// Note, in normal circumstances the 'pse@origin.osc.edu:bin/*'
	// would be taken in single or double quotes to avaid shell
	// metachar processing. In our case, the shell interpretor
	// is not used.
	// Return string with command, status, stdout, stderr


	Runtime rt = Runtime.getRuntime();
        if(scpOptions.length() > 0) {
	    String scpOptionsTrim = scpOptions.trim();
	    if(! scpOptionsTrim.startsWith("-")) {
		scpOptions = "-" + scpOptionsTrim;
	    } 
	}

	String scpCommand = "scp " + scpOptions + " " + localFile + " " +
	    user.trim() + "@" + host.trim() +  ":" + remoteFile.trim();
	System.err.println("copyFileToBackend  -- command: "+ scpCommand);

	String commandOutput = "COMMAND: " + scpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(scpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

}
