package ccm.pdt;
/**
 * @(#) MaintenanceBean.java
 * @author  Tom Haupt and Ken Flurchick
 * Date:    06/04/2000
 * Purpose: Problem Description Maintenance.
 *
 * Required Classes:
 *    Webflow 
 *    entry.Transform 
 *    entry.parseRU 
 */
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import WebFlow.*;
import WebFlow.event.*;
import WebFlow.ContextManager.*;
import org.omg.CORBA.*;
import java.util.*;
import java.text.*;
import entry.Transform;
import entry.parseRU;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.ProcessingInstruction;
/**
 * Class: MaintenanceBean
 * @param  None
 * @return None
 * @throws None
 */
  public class MaintenanceBean {
    ContextManager cm;
    String user; 
    String documentRoot;
    String problemName;
    String problemDescriptionName;
    String option;
    String message;
    boolean status = false;
    Document doc;
/**
 * Method: MaintenanceBean
 * <p>
 * Empty constructor.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public void MaintenanceBean(){
    // Empty constructor.
  }

// Setter methods for the Bean.
  public void setProblemDescriptionName(String S){
    this.problemDescriptionName = S;
    System.out.println("problemDescriptionName = "
      + problemDescriptionName);
  }
  public void setUser(String S){
    this.user = S;
  }
  public void setOption(String S){
    this.option = S;
  }
  public void setDocumentRoot(String S){
    this.documentRoot = S;
  }
// Getter methods for the Bean.
  public String getProblemDescriptionName(){
    return problemDescriptionName;
  }
  public String getUser(){
    return user;
  }
  public String getOption(){
    return option;
  }
  public String getDocumentRoot(){
    return documentRoot;
  }
/**
 * Method: parseOption
 * <p>
 * Parse on the options
 * </p>
 * <ol type="I">
 *   <li>"View" retrieve the existing problem description.
 *   <li>"Clone" retrieve the existing problem description.
 *   <li>"Delete" remove the existing problem description.
 *   <li>"Archive" store the problem description.
 * </ol>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean parseOption(){
    String PDstatus = "";
    cm = ServerServlet.getContextManager(user);
    if(option.equals("View"))    status = viewProblem();
    if(option.equals("Clone"))   status = cloneProblem();
    if(option.equals("Delete"))  status = deleteProblem(PDstatus);
    if(option.equals("Archive")) status = archiveProblem();
    return status;
  }
/**
 * Method: deleteProblem
 * <p>
 * Find the problem, delete it, and return to <em>listProblems</em>.
 * </p>
 * @param  String PDstatus
 * @return boolean status
 * @throws None
 */
  public boolean deleteProblem(String PDstatus) {
    boolean status = cm.removeProblem(problemDescriptionName);
    PDstatus = removePD();
    return status;
  }
/**
 * Method: viewProblem
 * <p>
 * Find the problem and display the XML file.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean viewProblem() {
    boolean status = false;
    String xmlFile = documentRoot + "WEB-INF/Descriptors/users/"
      + user + File.separator + problemDescriptionName + File.separator
      + problemDescriptionName + ".xml";
    String xslFile = documentRoot + "styles/PD.xsl";
    String htmlFile = documentRoot + "TempPages/viewProblem.html";
    Transform transformer = new Transform();
    status = transformer.writeHTML(xmlFile, xslFile, htmlFile);
    return status;
  }
/**
 * Method: archiveProblem
 * <p>
 * Find the problem and archive the XML file.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean archiveProblem() {
    return status;
  }
/**
 * Method: cloneProblem
 * <p>
 * Find the problem description and clone the XML file
 * sending the output to a form for the user to modify.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean cloneProblem() {
    return status;
  }
/**
 * Method: removePD
 * <p>
 * Remove the specified problem description PD.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public String removePD() {
    boolean status = false;
    String sStatus = "Failed";
    parseRU pf = new parseRU();

    String f1 = "MM  dd  yyyy  hh  mm";
    String f2 = "MM/dd/yyyy  hh:mm";
    SimpleDateFormat in = new SimpleDateFormat(f1);
    SimpleDateFormat out = new SimpleDateFormat(f2);
    Date curr;

    String fn = documentRoot + "WEB-INF/Descriptors/users/"
      + user + File.separator + user + ".xml";

    try {
      Calendar cal = Calendar.getInstance();
      String datestring = (
        cal.get(Calendar.MONTH) + 1) + "  "
          + cal.get(Calendar.DAY_OF_MONTH) + "  "
          + cal.get(Calendar.YEAR) + "  "
          + cal.get(Calendar.HOUR) + "  "
          + cal.get(Calendar.MINUTE);
      curr = in.parse(datestring);
    } catch (ParseException pe) {
      System.out.println("Error parsing date string");
      sStatus = "Error parsing date string";
      return sStatus;
    }
    long currentMillis = System.currentTimeMillis();

    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(fn);
    } catch (SAXException se) {
      System.out.println("removePD Errors > zero");
      sStatus =  "removePD Errors > zero";
      return sStatus;
    } catch (IOException ioe) {
      ioe.printStackTrace();
      sStatus =  "IOexception";
      return sStatus;
    }

    doc = parser.getDocument();
    pf.removeWhiteSpaceNodes(doc);
    pf.buildNodes(doc);
    Element r = doc.getDocumentElement();
    r.setAttribute("lastUpdate", out.format(curr));
    r.setAttribute("timeStamp", Long.toString(currentMillis));

    Vector aTags = pf.getAllTags(r, "problemName");
    int m = aTags.size();
    for (int j = 0; j < m; j++) {
      Node a = (Node)aTags.elementAt(j);
      if( ((Element)a).getAttribute("Name").equals(problemDescriptionName)) {
        r.removeChild(a);
        sStatus = "Success removing node.";
      }
    }

    // Save the XML document
    try {
      PrintWriter FileOut = new PrintWriter(
        new FileWriter(fn, false));
      OutputFormat format = new OutputFormat(doc, "UTF-8", true);
      XMLSerializer xmls = new XMLSerializer(FileOut, format);
      xmls.serialize(doc);
      sStatus += "Success writing user file!";
    } catch(IOException e){
      System.out.println("Error opening FileOut.");
      sStatus += "Error opening FileOut.";
    }
    return sStatus;
  }
/**
 * Method: getBeanInfo
 * @param       None
 * @return      None
 * @throws      None
 */
  public String getBeanInfo(){
    return "By K. Flurchick and T. Haupt";
  }
}
