package ccm.pdt;
/**
 * @(#) PDTnewBean.java
 * @author  Tom Haupt and Ken Flurchick
 * Date:    05/18/2000
 * Purpose: Bean for the "create new problem description"
 *          action of the Problem Description Toolkit.
 *
 * Required Classes:
 *          WebFlow Package
 */
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import WebFlow.*;
import WebFlow.event.*;
import WebFlow.ContextManager.*;
import org.omg.CORBA.*;
import java.util.*;
import java.util.Date.*;
import java.text.*;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.*;
import entry.parseRU.*;
/**
 * Class: PDTnew
 * <p>
 * This class will process the creation
 * of a new problem description.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public class PDTnewBean {
    ContextManager cm;
    String problemDescriptionName;
    String track;
    String user;
    String documentRoot;
    String message;
    String statusMessage;
    Element item;
/**
 * Method: PDTnewBean
 * <p>
 * Empty constructor.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public PDTnewBean() {
    // Empty constructor.
  }

// Setter methods for the Bean.
  public void setProblemDescriptionName(String S) {
    this.problemDescriptionName = S;
    System.out.println("problemDescriptionName = " 
      + problemDescriptionName);
  }
  public void setTrack(String S) {
    this.track = S;
  }
  public void setUser(String S) {
    this.user = S;
  }
  public void setDocumentRoot(String S) {
    this.documentRoot = S;
  }
  public void setMessage(String S) {
    this.message = S;
  }
// Getter methods for the Bean.
  public String getProblemDescriptionName() {
    return problemDescriptionName;
  }
  public String getTrack() {
    return track;
  }
  public String getUser() {
    return user;
  }
  public String getDocumentRoot() {
    return documentRoot;
  }
  public String getMessage() {
    message = statusMessage;
    return message;
  }

/**
 * Method: newProblem
 * <p>
 * This method will update the user.xml file and create 
 * the problem XML file.  
 * The <em>user</em> and <em>problemDescriptionName</em> are passed 
 * from the <em>pdtHandler.jsp</em> page.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean newProblem(){
/**
 * Dependent on site specific directory structure
 * and implied sciPortal structure.
 */
    String usersDir = documentRoot + "WEB-INF/Descriptors/users/" 
      + user + File.separator;
    String dataDir = documentRoot + "WEB-INF/Data/";
    boolean status;
    String Line;
    String cr, cr1;
/**
 * Create the file and directory names.
 */
    String pdtdFile = documentRoot + "dtd/pd.dtd";
    String problemDir = usersDir + problemDescriptionName;
    String problemFile = problemDir + File.separator 
      + problemDescriptionName + ".xml";
    String userFile = usersDir + user + ".xml";
/**
 * First, set the problem context and  allow the 
 * context manager to create the directory.
 */
    cm = ServerServlet.getContextManager(user);
    boolean cmStatus = cm.addProblem(problemDescriptionName);
  		 
    if(cmStatus){
      status = true;
      statusMessage = "cmStatus = " + cmStatus + "\n";
    } else {
      status = false;
      statusMessage = "cmStatus = " + cmStatus + "\n";
    }
/**
 * Now, set the pointer in the user.xml file.
 *
 * If the user XML file exists, open it and add the problem name.
 */
    if((cm != null) && cmStatus){
      // Add the entry to the userName.xml file.
      if(new File(userFile).exists()) {
        cr1 = updateUserFile();
        statusMessage += cr1;
        statusMessage += "Updated the user.xml \n\n";
      } else {
        cr1 = "User file does not exist - Entry Error! \n\n";
      }
      // Create the problemDescriptionName file.
      cr = createProblemDescription();
      statusMessage += "Returned from createProblemDescription()";
      statusMessage += "  cr = " + cr + "\n";
    }
    return status;
  }
/**
 * Method: updateUserFile
 * <p>
 * Update the 'user'.xml XML file.
 * </p>
 * @param  None
 * @return String status
 * @throws None
 */
  public String updateUserFile() {
/*
 * Now, update the 'user'.xml file.
 */
    String status;
    String pdPath = documentRoot + "WEB-INF/Descriptors/users/" 
      + user + File.separator + user + ".xml";

    try {
      String f1 = "MM  dd  yyyy  hh  mm";
      String f2 = "MM/dd/yyyy  hh:mm";
      SimpleDateFormat in = new SimpleDateFormat(f1);
      SimpleDateFormat out = new SimpleDateFormat(f2);

      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
        + cal.get(Calendar.DAY_OF_MONTH) + "  "
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + "  "
        + cal.get(Calendar.MINUTE);
      Date curr = in.parse(datestring);
      long currentMillis = System.currentTimeMillis();

      // Start parsing
      DOMParser parser = new DOMParser();
      parser.parse(pdPath);

      // Make <problemDescriptions> element root and add it
      // to the document.
      Document doc = parser.getDocument();
      Element root = doc.getDocumentElement();
      root.setAttribute("lastUpdate", out.format(curr));
      root.setAttribute("timeStamp", Long.toString(currentMillis));
  
      // Create the problemName tag
      item = doc.createElement("problemName");
      item.setAttribute("timeStamp", Long.toString(currentMillis));
      item.setAttribute("Name", problemDescriptionName);
      item.appendChild(doc.createTextNode(" "));

      root.insertBefore(item, null);

      try{
        PrintWriter fileOut = new PrintWriter(
          new FileWriter(pdPath, false));
        OutputFormat format = new OutputFormat(doc, "UTF-8", true);
        XMLSerializer serializer = new XMLSerializer(fileOut, format);
        serializer.serialize(doc);
      } // End of try block for FileOut.
      catch(IOException e){
        System.out.println("Error opening fileOut." + "   " + pdPath);
      }
      status = "User file updated.";
    } catch (Exception e) {
      status = "User file update error." + pdPath;
      e.printStackTrace();
    }
    return status;
  }
/**
 * Method: createProblemDescription
 * <p>
 * Create the 'problemDescriptionName'.xml XML file.
 * </p>
 * @param  None
 * @return String status
 * @throws None
 */
  public String createProblemDescription() {
/*
 * Now, create the 'problemDescriptionName'.xml file.
 */
    String status;
    String Tr = "User";
    String pdPath = documentRoot + "WEB-INF/Descriptors/users/" 
      + user + File.separator + problemDescriptionName
      + File.separator + problemDescriptionName + ".xml";

    if(new File(pdPath).exists()){
      status = "Error ... Problem description file already exists.";
      return status;
    } else {
      try {
        Document doc = new DocumentImpl();
        long currentMillis = System.currentTimeMillis();

        // Make <problemDescription> element root and add it
        // to the document.
        Element root = doc.createElement("problemDescription");
        root.setAttribute("timeStamp", Long.toString(currentMillis));

        item = doc.createElement("problemDescriptionName");
        item.appendChild(doc.createTextNode(problemDescriptionName));
        root.appendChild(item);

        item = doc.createElement("usertrack");
        if(track.equals("true")) {
          Tr = "User";
        } else if(track.equals("false")) {
          Tr = "Advanced";
        }
        item.appendChild(doc.createTextNode(Tr));
        root.appendChild(item);

        // Now root is added to the doc.
        doc.appendChild(root);

        try{
          PrintWriter fileOut = new PrintWriter(
            new FileWriter(pdPath, false));
          OutputFormat format = new OutputFormat(doc, null, true);
          format.setMethod("XML");
          format.setIndent(2);
          XMLSerializer serializer = new XMLSerializer(fileOut, format);
          serializer.serialize(doc);
        } // End of try block for fileOut.
        catch(IOException e){
          System.out.println("Error opening fileOut.");
        }
        status = "Problem Description File for " + problemDescriptionName 
          + "successfully created.";
      } catch (Exception e) {
        status = "Error in docImpl.";
        e.printStackTrace();
      }
      return status;
    }
  }
/**
 * Method: getBeanInfo
 * @param  None
 * @return None
 * @throws None
 */
  public String getBeanInfo() {
    return "By K. Flurchick and T. Haupt";
  }
}
