package ccm.pdt;
/**
 * @(#) PDTparser.java
 * @author 	Ken Flurchick <kenf@osc.edu>
 * Date:	05/19/2000
 * Purpose:	Parse the user.xml and problem XML files.
 */
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import java.io.*;
import java.text.*;
import java.util.*;
import java.util.Date.*;
import org.w3c.dom.*;
/**
 * Class: PDTparser
 *<p>
 *Methods
 *</p>
 *<ul type="square">
 *  <li>createProblemDescription(String userName, String problemName)
 *      <br />- Create the user XML file named by <em>userName</em>
 *  <li>updateProblemDescription(String userName, String problemName)
 *      <br />- Update the values of the user XML file named by <em>userName</em>
 *</ul>
 * @param  None
 * @return None
 * @throws None
 */
public class PDTparser {
  Vector nVec = new Vector(); // nVec is the key vector.
  Vector vVec = new Vector(); // vVec is the value vector.
  static String value = null;
  Hashtable tHash = new Hashtable();
  Hashtable hash = new Hashtable();
  Vector userKey = new Vector();
  Vector userValue = new Vector();
  int nEntries = 0;
  Element item, itemA, itemB;
  String message = " Parser message :   ";
/**
 * Constructor: PDTparser
 * @param	None
 * @return	None
 * @throws	None
 */
  public void PDTparser(){
    // Empty Constructor
  }
/**
 * Method: createUserXML
 *<p>
 *Creates the XML document to update the 'userName'.xml
 *XML file.
 *</p>
 * @param  String userDir
 * @param  String xmlFile
 * @param  String dtdFile
 * @param  String userName
 * @param  String problemName
 * @return boolean Status
 * @throws None
 */
  public String createUserXML(String Dir, String xFile,
    String dtdFile, String userName, String problemName){
    boolean status = true;
    String pdPath = Dir + File.separator + userName + File.separator
      + problemName + File.separator + problemName + ".xml";
    message += "xFile = " + xFile;
    try {
      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "/"
        + cal.get(Calendar.DAY_OF_MONTH) + "/"
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + ":"
        + cal.get(Calendar.MINUTE);
      long currentMillis = System.currentTimeMillis();
      String cmS = Long.toString(currentMillis);

      // Check to see whether an xml file exists.  If not, make one:
      try {
        FileReader fr = new FileReader(xFile); 
        fr.close();
      }
      catch ( FileNotFoundException e) {
        try {
          // No file found.  Create a skeleton file and write it out
          PrintWriter pw = new PrintWriter(new FileWriter(xFile));
          pw.println("<?xml version =\"1.0\" standalone=\"no\" ?>\n");
          pw.println("<!DOCTYPE problemDescriptions SYSTEM " + dtdFile + ">");
          pw.println("<problemDescriptions timeStamp=\"" + cmS + "\">");
          pw.println("</problemDescriptions>");
          pw.flush();
          pw.close(); 
        }
        catch(IOException fnfe){
          System.out.println("Error opening fileOut1.");
          status = false;
          message += "error opening fileOut " + fnfe + "\n";
          return message;
        }
      }

      // Open xml file and Start parsing
      DOMParser parser = new DOMParser();
      parser.parse(xFile);

      // Get the root element <problemDescriptions>
      // and set the attribute.
      Document doc = parser.getDocument();
      Element r = doc.getDocumentElement();
      r.setAttribute("lastUpdate", datestring);
      r.setAttribute("timeStamp", "");
  
      //  Add a  <problemDescription> element and add it
      //  as a child of root
      item = doc.createElement("problemName");
      itemA.appendChild(doc.createTextNode(pdPath));
      item.appendChild(itemA);

      // Now problem is added to the root.
      r.appendChild(item);
  
      // Now root is added to the doc.
      doc.appendChild(r);

      try{
        PrintWriter fileOut = new PrintWriter(
          new FileWriter(xFile, false));
        OutputFormat format = new OutputFormat(doc, null, true);
        XMLSerializer serializer = new XMLSerializer(fileOut, format);
        serializer.serialize(doc);
      }
      catch(IOException e){
        System.out.println("Error opening fileOut2.");
        status = false;
        message += "error opening fileOut " + e + "\n";
        return message;
      }
      status = true;
    } catch (Exception e) {
      status = false;
      e.printStackTrace();
      message += "error in parser " + e;
      return message;
    }
    message += status;
    return message;
  }
/**
 * Method: createProblemDescription
 *<p>
 *Use TX XML document to create the 'problemName'.xml
 *XML file.
 *</p>
 * @param  String xmlFile
 * @param  String dtdFile
 * @param  String userName
 * @param  String problemName
 * @return boolean Status
 * @throws None
 */
  public boolean createProblemDescription(String Dir, String xmlFile, String dtdFile,
    String userName, String problemName){
/*
 * Now, create the 'problemName'.xml file.
 */
    boolean status;
    String pdPath = Dir + File.separator + problemName
      + File.separator + problemName + ".xml";

    try {
      String f1 = "MM  dd  yyyy  hh  mm";
      String f2 = "MM/dd/yyyy  hh:mm";
      SimpleDateFormat in = new SimpleDateFormat(f1);
      SimpleDateFormat out = new SimpleDateFormat(f2);
    
      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
        + cal.get(Calendar.DAY_OF_MONTH) + "  "
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + "  "
        + cal.get(Calendar.MINUTE);
      Date curr = in.parse(datestring);

      // Check to see whether an xml file exists.  If not, make one:
      try {
        FileReader fr = new FileReader(pdPath); 
        fr.close();
      }
      catch ( FileNotFoundException e) {
        try {
          // No file found.  Create a skeleton file and write it out
          PrintWriter pw = new PrintWriter(new FileWriter(pdPath));
          pw.println("<?xml version =\"1.0\" standalone=\"no\" ?>\n");
          pw.println("<!DOCTYPE problem SYSTEM \"http://SciPortal/dtd/pd.dtd\">");
          pw.println("<problem name=\"\" version=\"\">");
          pw.println("  <application name=\"\" version=\"\">");
          pw.println("    <inputFile></inputFile>");
          pw.println("    <outputFile></outputFile>");
          pw.println("    <type></type>");
          pw.println("  </application>");
          pw.println("</problem>");
          pw.flush();
          pw.close();
        }
        catch(IOException fnfe){
          System.out.println("Error opening fileOut3.");
          return false;
        }
      }

      // Start parsing
      DOMParser parser = new DOMParser();
      parser.parse(pdPath);

      // Make <problemDescriptions> element root and add it
      // to the document.
      Document doc = parser.getDocument();
      Element root = doc.createElement("problemDescriptions");
      root.setAttribute("timeStamp", out.format(curr));

      //  Make <problem> element and add it
      //  as a child of root
      item = doc.createElement("problemDescription");
      itemB.appendChild(doc.createTextNode(problemName));

      //  Make <application> element and add it
      //  as a child of problem
      itemA = doc.createElement("application");
      itemA.setAttribute("name", "Application Name");
      itemA.setAttribute("version", "Version Number");
      item.appendChild(itemA);

      //  Make <inputFile> element and add it
      //  as a child of problem
      itemB = doc.createElement("inputFile");
      itemB.appendChild(doc.createTextNode("Input File name"));
      itemA.appendChild(itemB);

      //  Make <outputFile> element and add it
      //  as a child of problem
      itemB = doc.createElement("outputFile");
      itemB.appendChild(doc.createTextNode("Output File name"));
      itemA.appendChild(itemB);

      //  Make <type> element and add it
      //  as a child of problem
      itemB = doc.createElement("type");
      itemB.appendChild(doc.createTextNode("Type of calculation"));
      itemA.appendChild(itemB);
  
      // Now problem is added to the root.
      root.appendChild(item);
  
      // Now root is added to the doc.
      doc.appendChild(root);

      try{
        PrintWriter fileOut = new PrintWriter(
          new FileWriter(pdPath, false));
        OutputFormat format = new OutputFormat(doc, null, true);
        XMLSerializer serializer = new XMLSerializer(fileOut, format);
        serializer.serialize(doc);
      } // End of try block for ruFileOut.
      catch(IOException e){
        System.out.println("Error opening fileOut4.");
      }
      status = true;
    } catch (Exception e) {
      status = false;
      e.printStackTrace();
    }
    return status;
  }
/**
 * Method: keyVector
 *<p>
 *Generate a vector of the list of tags.
 *</p>
 * @param  None
 * @return None
 * @throws None
 */
  void keyVector(){
    userKey.add("problemDescriptions");
    userKey.add("timeStamp");
    userKey.add("problem");
    userKey.add("name");
    userKey.add("application");
    userKey.add("version");
    userKey.add("inputFile");
    userKey.add("outputFile");
    userKey.add("type");
  }
/** 
===============================================================================
===============================================================================
 *
 * Method: removeWhiteSpaceNodes
 *<p>
 *Remove the white space associated with a node.
 *</p>
 * @param  Element parent
 * @return None
 * @throws None
 */
  private void removeWhiteSpaceNodes ( Node parent ) {
    Node nextNode = parent.getFirstChild();
    for (Node child = parent.getFirstChild(); nextNode != null;) {
      child = nextNode;
      nextNode = child.getNextSibling();
      int type = child.getNodeType();

      if (Node.TEXT_NODE != type) {
        removeWhiteSpaceNodes(child);
      }
      else {
        if (((Text) child).getData().trim().length() == 0) {
          parent.removeChild(child);
        }
      }
    }
  }

/**
 * Method: buildNodes
 *<p>
 *Parse a node and return the name/value pairs
 *as two vectors:
 *</p>
 *<ol>
 *  <li>nVec - vector of keys.
 *  <li>vVec - vector of values
 *</ol>
 * @param  NodeList nl
 * @param  String hName
 * @return None
 * @throws None
 */
  public void buildNodes(NodeList nl, String hName){
    int inl = 0;
    int inl2 = 0;
    int l;
    boolean found = false;

    while(inl < (nl.getLength())){
      Node childNL = nl.item(inl);
      String chName = childNL.getNodeName();
      if(chName != "#text") {
        hName = chName;
      }
      String chValue = childNL.getNodeValue();
      if (childNL.getNodeType() == Node.TEXT_NODE) {
        chValue = childNL.getNodeValue();
      } else if (childNL.hasChildNodes()){
        NodeList nl2 = childNL.getChildNodes();
        buildNodes(nl2, hName);
      }
      inl++;
      if((hName != null) && (chValue != null)) {
        if(nVec.isEmpty() == true){
          nVec.addElement(hName);
          vVec.addElement(chValue);
          found = false;
        } else if(nVec.isEmpty() != true){
          for(l = 0; l < nVec.size(); l++){
            if (hName.equals(nVec.get(l))){
              found = true;
              value = (String)vVec.get(l);
              value = value + "::" + chValue;
              vVec.setElementAt(value, l);
              break;
            }
          }
          if(found == false) {
            nVec.addElement(hName);
            vVec.addElement(chValue);
            found = false;
          }
        }
      }
    }
  }
/**
 * Method: getAlltags
 *<p>
 *Get a Vector of nodes that have a specific node name.
 *</p>
 * @param  Node node
 * @param  String tag
 * @return Vector result
 * @throws None
 */
  private Vector getAllTags(Node node, String tag) {
    Vector result=new Vector();

    for(Node ch = node.getFirstChild(); ch!=null; ch = ch.getNextSibling()) {
      if(ch instanceof Element) {
        if(ch.getNodeName().equals(tag)) {
          result.addElement(ch);
        }
      }
    }
    return result;
  }
}
