package ccm.pdt;
/**
 * @(#)Transform
 * @author  Wade Lutgen <wade@pacific.mps.ohio-state.edu>
 * Date:    07/19/2000
 * Purpose: Given an XML and XSL file, this program creates 
 *          an HTML file.
 *
 * Required Classes: none
 */
import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xalan.*;
import java.lang.reflect.*;
import org.apache.xalan.xslt.*;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.xml.*;
/**
 * Class: Transform 
 * <p>
 * This is the main class for <strong>Transform</strong>.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
public class Transform {
/**
 * Constructor: Transform 
 * <p>
 * Empty Constructor 
 * </p>
 * @param  None 
 * @return None
 * @throws None 
 */
  Transform() {
    // Empty Constructor
  }
/**
 * Constructor: Transform
 * <p>
 * This constructor will take a xml file, a xsl file, and an 
 * html file and call writeHTML().
 * </p>
 * @param  String xmlFileName - input file.  Must be xml
 * @param  String xslFileName - input file.  Must be xsl
 * @param  String htmlFileName - output file.  This is html 
 * @return None
 * @throws None
 */
  Transform(String xmlFileName, String xslFileName, String htmlFileName) {
    writeHTML(xmlFileName, htmlFileName, xslFileName);
  }
/**
 * Method: writeHTML
 * <p>
 * This member parses the inputs and writes the output file 
 * </p>
 * @param  String xmlFileName - input file.  Must be xml
 * @param  String xslFileName - input file.  Must be xsl
 * @param  String htmlFileName - output file.  This is html 
 * @return None
 * @throws None
 */
  public boolean writeHTML(String xmlFileName, String xslFileName, String htmlFileName){
    boolean status = false;

    XMLParserLiaison xmlProcessorLiaison = null;
    String parserLiaisonClassName = Constants.LIAISON_CLASS;
    try {
      Class parserLiaisonClass = Class.forName(parserLiaisonClassName);
      Constructor parserLiaisonCtor = parserLiaisonClass.getConstructor(null);
      xmlProcessorLiaison=(XMLParserLiaison)parserLiaisonCtor.newInstance(null);
    }
    catch(Exception e) {
      System.err.println("Could not create XML Processor Liaison....exiting");
      status = false;
    }

    XSLTProcessor processor = XSLTProcessorFactory.getProcessor(xmlProcessorLiaison);

    xmlProcessorLiaison.setIndent(2);
    // xmlProcessorLiaison.setUseValidation(true);

    try {
      StylesheetRoot stylesheet = processor.processStylesheet(xslFileName);
      OutputStream outputStream = new FileOutputStream(htmlFileName);
      stylesheet.setOutputMethod("html");
      stylesheet.m_useXercesSerializers = true;
      Node sourceTree = processor.getSourceTreeFromInput(
                        new XSLTInputSource((xmlFileName)));
      stylesheet.process(processor, sourceTree, 
                         new XSLTResultTarget(outputStream));
      status = true;
    }
    catch(SAXException se) {
      System.err.println("SAXException.  Exiting");
      status = false;
    }
    catch(FileNotFoundException fne) {
      System.err.println("File not found.  Exiting");
      status = false;
    }
    catch(IOException ioe) {
      System.err.println("IO excepction.  Exiting.");
      status = false;
    }
    catch(java.lang.NoClassDefFoundError ncdfe) {
      System.err.println("A Class was not found. Exiting");
      status = false;
    }
    catch(RuntimeException rte) {
      System.err.println("Runtime error.  Exiting");
      status = false;
    }
    return status;
  }
}
