package ccm.test;

//
// File:   FileUploadServlet.java
// Author: Christopher Hoover <ch@murgatroid.com>
// $Id$
//
import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.mail.*;
import javax.mail.internet.*;

public class FileUploadServlet extends HttpServlet {

    final static int maximumUploadLength = 10000000;
    
    public void init(ServletConfig config) throws ServletException {
	super.init(config);
    }
    
    public void doPost(HttpServletRequest request, HttpServletResponse
		       response) throws ServletException, IOException  {
	if (request.getContentLength() > maximumUploadLength) {
	    System.err.println("Content too long: " + 
			       request.getContentLength());
	    response.sendError(response.SC_BAD_REQUEST, "Content too long.");
	    return;
	}
	
	if (request.getContentType().toLowerCase().equals("multipart/form-data"))   {
	    System.err.println("Bad content type: "+request.getContentType());
	    response.sendError(response.SC_BAD_REQUEST, "Bad content type.");
	    return;
	    }
	
	try {
	    Properties props = System.getProperties();
	    Session session = Session.getDefaultInstance(props, null);
	    
	    String headers = "Content-Type: " + request.getContentType() + 
		"\r\n\r\n";
	    InputStream messageIS =
		new SequenceInputStream(new  StringBufferInputStream(headers),
					request.getInputStream());
	    MimeMessage message = new MimeMessage(session, messageIS);

	    Object content = message.getContent();
	    
	    if (content instanceof MimeMultipart) {
		MimeMultipart mm = (MimeMultipart) content;
		
		int nBodyParts = mm.getCount();
		for (int i = 0; i < nBodyParts; i++) {
		    BodyPart part = mm.getBodyPart(i);
		    
		    System.out.println("Body part #" + i);
		    System.out.println("content type   :" + 
				       part.getContentType());
		    System.out.println("description    :" + 
				       part.getDescription());
		    System.out.println("file name      :" + 
				       part.getFileName());
		    System.out.println("size           :" + 
				       part.getSize());
		    System.out.println("line count     :" + 
				       part.getLineCount());
		}
	    } else
		System.err.println("Content is not MimeMultipart");
	} catch (MessagingException e) {
	    System.err.println("Failed to parse content: " + e);
	    response.sendError(response.SC_BAD_REQUEST,
			       "Failed to parse content.");
	}
	
	response.setContentType("text/plain");
	PrintWriter out = response.getWriter();
	out.println("You won big!");
    }
}
