package entry;
/**
 * @(#) EntryBean.java
 * @author  Ken Flurchick
 * Date:    04/22/2000
 * Purpose: This bean handles the SciPortal welcome page.
 * Needed Classes: parseRU
 */
import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.*;
import java.text.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.ProcessingInstruction;

import entry.parseRU.*;
/**
 * Class: EntryBean
 * <p>
 * This class will read the registered user information file
 * for the users login.
 * </p>
 * <p>
 * The document root is the SciPortal level.
 * <br />
 * <em>"/usr/local/apache_t3.1/htdocs/SciPortal/"</em>
 * </p>
 * <p>
 * The setter method constructs a userName string.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
public class EntryBean {
  String userName;
  String htmlFile;
  String uName = "inValid";
  String user;
  String documentRoot; 

  public EntryBean(){
    // Empty constructor.
  }

// Setter methods for the Bean.
  public void setUser(String S){
    this.user = S;
  }
  public void setDocumentRoot(String S){
    this.documentRoot = S;
  }
  public void setHtmlFile(String S){
    this.htmlFile = S;
  }
// Getter methods for the Bean.
  public String getUser(){
    return user;
  }
  public String getDocumentRoot(){
    return documentRoot;
  }
  public String getHtmlFile(){
    return htmlFile;
  }
/**
 * Method: viewProfile
 * <p>
 * Find the profile.xml and display the XML file.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean viewProfile() {
    boolean status = false;
    String xmlFile = documentRoot + "WEB-INF/Descriptors/users/"
      + user + "/profile.xml";
    String xslFile = documentRoot + "WEB-INF/Data/profile.xsl";
    htmlFile = documentRoot + "TempPages/profile.html";
    Transform transformer = new Transform();
    status = transformer.writeHTML(xmlFile, xslFile, htmlFile);
    return status;
  }
/**
 * Method: getUserName
 * <p>
 * Obtain the users full name.
 * </p>
 * @param  None
 * @return String userName
 * @throws None
 */
  public String getUserName() {
    String status = null;
    parseRU pf = new parseRU();

    status = pf.readRU(user, documentRoot);
    // status = "user = " + user + "  rootDir = " + documentRoot;
    return status;
  }
/**
 * Method: makeProfile
 * <p>
 * Write the <em>profile.xml</em> file.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean makeProfile(){
    boolean status;

    parseRU pf = new parseRU();
    status = pf.createProfile(user, documentRoot);
    return status;
  }
/**
 * Method: makeUserFile()
 * <p>
 * Write the user profile from the registeredUser.xml.
 * </p>
 * @param  None
 * @return String status
 * @throws None
 */
  public String makeUserFile() {
    String status = "";
    Element root, item, itemA, itemB;
    String buildkey;
    String userFile = documentRoot + "WEB-INF/Descriptors/users/"
      + user + "/" + user + ".xml";

    long currentMillis = System.currentTimeMillis();

    if(new File(userFile).exists()){
      status = "User file already exists.";
      return status;
    } else {
      // Now, begin creating the user.xml file.
      try {
        Document doc = new DocumentImpl();

        String f1 = "MM  dd  yyyy  hh  mm";
        String f2 = "MM/dd/yyyy  hh:mm";
        SimpleDateFormat in = new SimpleDateFormat(f1);
        SimpleDateFormat out = new SimpleDateFormat(f2);

        Calendar cal = Calendar.getInstance();
        String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
          + cal.get(Calendar.DAY_OF_MONTH) + "  "
          + cal.get(Calendar.YEAR) + "  "
          + cal.get(Calendar.HOUR) + "  "
          + cal.get(Calendar.MINUTE);
        Date curr = in.parse(datestring);

        // Make <problemNames> element root and add it
        // to the document.
        root = doc.createElement("problemNames");
        root.setAttribute("lastUpdate", out.format(curr));
        root.setAttribute("timeStamp", Long.toString(currentMillis));

        Text tn = doc.createTextNode(" ");
        root.appendChild(tn);

        // Now root is added to the doc.
        doc.appendChild(root);
  
        System.out.println("userFile = " + userFile);
        // Save the XML document
        try {
          PrintWriter FileOut = new PrintWriter(new FileWriter(userFile, false));
          OutputFormat format = new OutputFormat(doc, "UTF-8", true);
          XMLSerializer xmls = new XMLSerializer(FileOut, format);
          xmls.serialize(doc);
          FileOut.close();
          status = "User file created.";
        } catch(IOException e){
          status = "Error IOException.";
          System.out.println("Error opening FileOut.");
        }
      } catch (Exception e) {
        status = "Print error";
        e.printStackTrace();
      }
      return status;
    }
  }
}
