package entry;
/**
 * @(#)parseRU.java
 * @author  Ken Flurchick <kenf@osc.edu>
 * Date:    02/27/2000
 * Purpose: Parse the registeredUser XML file
 *          using the Xerces API.
 * @version 0.75 Removed the vector, pass  back 
 *               only the hash table.
 * @version 0.01 Initial version.
 */
import java.io.*;
import java.util.*;
import java.text.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.ProcessingInstruction;
/**
 * Class: parseRU()
 * <p>
 * Methods
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
public class parseRU {
  Vector nVec = new Vector(); // nVec is the key vector.
  Vector vVec = new Vector(); // vVec is the value vector.
  static String value = null;
  Hashtable tHash = new Hashtable();
  Hashtable hash = new Hashtable();
  Vector userKey = new Vector();
  Vector userValue = new Vector();
  int nUsers = 0;
  int startIndex, endIndex;
  Element root, item, itemA, itemB;
  String buildkey;
  Document doc;
  String ruFile, dtdFile;
/**
 * Constructor: parseRU
 * <p>
 * Empty constructor.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public parseRU(){
    // empty constructor
  }
/**
 * Method: doParse
 * <ul type="square">
 *   <li>0 - OK
 *   <li>1 - Missing file name
 *   <li>2 - Parse errors
 * </ul>
 * <p>
 * Parse an XML file and return the hashtable of 
 * keys and values of the entire file not ordered.
 * </p>
 * @param  String name 
 * @param  String dtdFile
 * @param  Hashtable hash
 * @return integer
 * @throws None
 */
  public int doParse(String name, String dtdFile, Hashtable hash){
    String key = null, value = null;
    String hName = null;
    String lup = null;
    nUsers = 0;

    if (name == null || dtdFile == null) {
      System.err.println("Missing filename.");
      return nUsers;
    }
       
    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println("doParse Errors > zero");
      return(2);
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return(2);
    }

    // Document is well-formed

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    lup = r.getAttribute("lastUpdate");
    // System.out.println("doParse lup = " + lup);
    Vector aTags = getAllTags(r, "User");
    nUsers = aTags.size();

    // Make the hash table from the name/value pairs.
    hash.put("lastUpdate", lup);
    for(int l = 0; l < nVec.size(); l++){
      hash.put(nVec.get(l), (vVec.get(l) + "::"));
    }
    //System.out.println("Hash Table = " + hash.toString());
    return nUsers;
  }
/**
 * Method: doParseF
 * <p>
 * Parse an XML file and return the hashtable of keys
 * and values for a specific user.
 * </p>
 * @param  String name
 * @param  String dtdFile
 * @param  Hashtable thash
 * @return integer
 * @throws None
 */
  public int doParseF(String name, String dtdFile, Hashtable tHash){
    nUsers = 0;
    String key = null, value = null, hName = null;

    if (name == null || dtdFile == null) {
      System.err.println("Missing filename.");
      return nUsers;
    }

    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println("doParseF Errors > zero");
      return(2);
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return(2);
    }
    // Document is well-formed

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    Vector aTags = getAllTags(r, "User");
    nUsers = aTags.size();

    // Make the hash table from the name/value pairs.
    for(int l = 0; l < nVec.size(); l++){
      hash.put(nVec.get(l), (vVec.get(l) + "::"));
    }
    //System.out.println("Hash Table = " + hash.toString());

    keyVector();
    // Make the tHash table (assembled by User ID rather
    // than by keywords) from the name/value pairs.
    // Zeroth element is the number of users.
    // System.out.println("nUsers = " + nUsers);
    for(int k = 0; k < userKey.size(); k++){
      startIndex = 0;
      for(int l = 0; l < nUsers; l++){
        if(k == 0){
          String tt = (String)hash.get(userKey.get(k));
          userValue.add(l, tt.substring(startIndex,
            tt.indexOf("::", startIndex)) + "::");
          startIndex = tt.indexOf("::", startIndex) + 2;
        } else {
          String tt = (String)hash.get(userKey.get(k));
          userValue.set(l, (String)userValue.elementAt(l)
            + tt.substring(startIndex,
            tt.indexOf("::", startIndex)) + "::");
          startIndex = tt.indexOf("::", startIndex) + 2;
        }
      }
    }
    for(int l = 0; l < nUsers; l++){
      String XX = (String)userValue.get(l);
      tHash.put(XX.substring(0, XX.indexOf("::")), XX);
    }
    return nUsers;
  }
/**
 * Method: readRU
 * <p>
 * Read the registered user information file.
 * </p>
 * @param  String UID
 * @param  String documentRoot
 * @return String uName
 * @throws None
 */
  public String readRU(String UID, String documentRoot) {
    String usrs = null;
    String focus = null;
    String uName = null;
    Hashtable uHash = new Hashtable();
    Hashtable userHash = new Hashtable();
    String profileFile = null;

    // System.out.println("Find the user from the = " + ruFile + " file.");
    ruFile = documentRoot + "WEB-INF/Data/rUser.xml";
    dtdFile = documentRoot + "WEB-INF/Data/rUser.dtd";
    nUsers = doParseF(ruFile, dtdFile, userHash);

    if(nUsers == 0){
      System.out.println("Error in generating hash table");
      return "Error in generating hash table";
    }
    // System.out.println("Parsed ruFile = " + ruFile);
    // System.out.println("nUsers = " + nUsers);
    // System.out.println("usrs = " + userHash.toString());
    // System.out.println("user key name = " + UID);

    usrs = (String)userHash.get(UID);

    // System.out.println("usrs = " + usrs);
    try
    {
        keyVector();
        if(usrs != null){
          // System.out.println("in usrs not null - usrs = " + usrs);

          startIndex = 0;
          for(int i = 0; i < userKey.size(); i++ ){
            endIndex = usrs.indexOf("::", startIndex);

            uHash.put(userKey.get(i), usrs.substring(startIndex, endIndex));
            startIndex = endIndex + 2;
          }

          uName = (String)uHash.get("firstName") + "  "
            + "  " + (String)uHash.get("middleName")
            + "  " + (String)uHash.get("lastName");
            // System.out.println("uName = " + uName);

        } else {
          //System.out.println("ERROR: user name is invalid!");
          //return usrs;
          return "Error: user name " + usrs + " is invalid";
        }
    } catch (NullPointerException npe) {
      uName = "ERROR: Attempting to add null node to hashtable";
    } catch (IndexOutOfBoundsException ioobe) {
      uName = "ERROR: Attempting to access substring out of bounds";
    }

    return uName;
  }
/**
 * Method: createProfile()
 * <p>
 * Create the user profile from the registeredUser.xml.
 * </p>
 * @param  String UID
 * @param  String documentRoot
 * @return boolean 
 * @throws None
 */
  public boolean createProfile(String UID, String documentRoot) {
    boolean status = false;
    String usrs;
    Hashtable uData = new Hashtable();
    Hashtable userData = new Hashtable();
    Hashtable userHash = new Hashtable();

    String profileFile = documentRoot + "WEB-INF/Descriptors/users/"
      + UID + "/profile.xml";
/**
 * Get the hash table of the registered user information file.
 * Use doParseF to get the list ordered by the user,
 * Grab the entries for the specified UID.
 * Use this to build the profile.
 */
    // System.out.println("Find the user from the = " + ruFile + " file.");
    ruFile = documentRoot + "WEB-INF/Data/rUser.xml";
    dtdFile = documentRoot + "WEB-INF/Data/rUser.dtd";
    nUsers = doParseF(ruFile, dtdFile, userHash);

    usrs = (String)userHash.get(UID);
    keyVector();
    for(int i = 0, startIndex = 0; i < userKey.size(); i++ ){
      endIndex = usrs.indexOf("::", startIndex);
      uData.put(userKey.get(i), usrs.substring(startIndex, endIndex));
      startIndex = endIndex + 2;
    }
/*
 * Now, begin creating the profile.xml file.
 */
    try {
      Document doc = new DocumentImpl();

      String f1 = "MM  dd  yyyy  hh  mm";
      String f2 = "MM/dd/yyyy  hh:mm";
      SimpleDateFormat in = new SimpleDateFormat(f1);
      SimpleDateFormat out = new SimpleDateFormat(f2);

      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
        + cal.get(Calendar.DAY_OF_MONTH) + "  "
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + "  "
        + cal.get(Calendar.MINUTE);
      Date curr = in.parse(datestring);

      // Make <registeredUser> element root and add it
      // to the document.
      root = doc.createElement("registeredUser");
      root.setAttribute("lastUpdate", out.format(curr));

      // Make <User> element and add it
      // as a child of root
      item = doc.createElement("User");
      item.setAttribute("UID", (String)uData.get("userID"));

      // Make <primaryArea> element and add it
      // as a child of User
      itemA = doc.createElement("primaryArea");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("primaryArea")));
      item.appendChild(itemA);

      // Make <projectNumber> element and add it
      // as a child of User
      itemA = doc.createElement("projectNumber");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("projectNumber")));
      item.appendChild(itemA);

      // Make <email> element and add it
      // as a child of User
      itemA = doc.createElement("email");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("email")));
      item.appendChild(itemA);

      // Make <organization> element and add it
      // as a child of User
      itemA = doc.createElement("organization");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("organization")));
      item.appendChild(itemA);

      // Make <Person> element and add it
      // as a child of User
      itemA = doc.createElement("Person");

      // Make <title> element and add it
      // as a child of Person
      itemB = doc.createElement("title");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("title")));
      itemA.appendChild(itemB);
  
      // Make <lastName> element and add it
      // as a child of Person
      itemB = doc.createElement("lastName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("lastName")));
      itemA.appendChild(itemB);

      // Make <firstName> element and add it
      // as a child of Person
      itemB = doc.createElement("firstName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("firstName")));
      itemA.appendChild(itemB);

      // Make <middleName> element and add it
      // as a child of Person
      itemB = doc.createElement("middleName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("middleName")));
      itemA.appendChild(itemB);

      // Add <Person> as a child of <user>
      item.appendChild(itemA);

      // Make <Address> element and add it
      // as a child of User
      itemA = doc.createElement("Address");
  
      // Make <street1> element and add it
      // as a child of Address
      itemB = doc.createElement("street1");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("street1")));
      itemA.appendChild(itemB);

      // Make <street2> element and add it
      // as a child of Address
      itemB = doc.createElement("street2");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("street2")));
      itemA.appendChild(itemB);

      // Make <mailStop> element and add it
      // as a child of Address
      itemB = doc.createElement("mailStop");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("mailStop")));
      itemA.appendChild(itemB);

      // Make <building> element and add it
      // as a child of Address
      itemB = doc.createElement("building");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("building")));
      itemA.appendChild(itemB);

      // Make <city> element and add it
      // as a child of Address
      itemB = doc.createElement("city");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("city")));
      itemA.appendChild(itemB);

      // Make <state> element and add it
      // as a child of Address
      itemB = doc.createElement("state");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("state")));
      itemA.appendChild(itemB);

      // Make <zipcode> element and add it
      // as a child of Address
      itemB = doc.createElement("zipcode");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("zipcode")));
      itemA.appendChild(itemB);

      // Make <zip4> element and add it
      // as a child of Address
      itemB = doc.createElement("zip4");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("zip4")));
      itemA.appendChild(itemB);

      // Add <Address> as a child of <user>
      item.appendChild(itemA);

      // Make <contact> element and add it
      // as a child of User
      itemA = doc.createElement("contact");
  
      // Make <officePhone> element and add it
      // as a child of contact
      itemB = doc.createElement("officePhone");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("officePhone")));
      itemA.appendChild(itemB);

      // Make <faxNumber> element and add it
      // as a child of contact
      itemB = doc.createElement("faxNumber");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("faxNumber")));
      itemA.appendChild(itemB);

      // Add <contact> as a child of <user>
      item.appendChild(itemA);

      // Make <machine> element and add it
      // as a child of user
      itemA = doc.createElement("Machine");

      itemB = doc.createElement("machineID");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("machineID")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("homeDir");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("homeDir")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("queues");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("queues")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("UID");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("UID")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("group");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("group")));
      itemA.appendChild(itemB);
  
      item.appendChild(itemA);
  
      // Make <code> element and add it
      // as a child of user
      itemA = doc.createElement("code");
  
      itemB = doc.createElement("codeName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("codeName")));
      itemA.appendChild(itemB);
  
      item.appendChild(itemA);

      // Add User to the root element
      root.appendChild(item);

      // Now root is added to the doc.
      doc.appendChild(root);

      System.out.println("name = " + profileFile);
      // Save the XML document
      try {
        PrintWriter FileOut = new PrintWriter(
          new FileWriter(profileFile, false));
        OutputFormat format = new OutputFormat(doc, "UTF-8", true);
        XMLSerializer xmls = new XMLSerializer(FileOut, format);
        xmls.serialize(doc);
        status = true;
      } catch(IOException e){
        status = false;
        System.out.println("Error opening FileOut.");
      }
    } catch (Exception e) {
      status = false;
      e.printStackTrace();
    }
    System.out.println("status = " + status + "  fileName = " 
      + profileFile);
    if(status){
      return status;
    } else {
      status = false;
      return status;
      // return "In parseRU - createProfile status = " + status
      //  + "  fileName = " + profileFile;
    }
  }
/**
 * Method: removeWhiteSpaceNodes
 * <p>
 * Remove the white space associated with a node.
 * </p>
 * @param  Element parent
 * @return None
 * @throws None
 */
  public void removeWhiteSpaceNodes(Node parent) {

    Node nextNode = parent.getFirstChild();

    // System.out.println(parent.getNodeName() + " " + 
    // parent.getChildNodes().getLength() + " " + parent.getNodeValue());

    // Deal with element nodes with no children 
      for (Node child = parent.getFirstChild(); nextNode != null;){
        child = nextNode;
        nextNode = child.getNextSibling();
        int type = child.getNodeType();

        if (Node.TEXT_NODE != type){
          removeWhiteSpaceNodes(child);
        } else if (child.getNodeValue().trim().length() == 0) {
          parent.removeChild(child);
        }
      }

      // If an element doesn't have a child at this point, it needs one
      if(parent.getNodeType() == Node.ELEMENT_NODE &&
        parent.getChildNodes().getLength() == 0) {
        // Add a dummy text node
        Text tn = doc.createTextNode(" ");
        parent.appendChild(tn);
      }
  }
/**
 * Method: buildNodes
 * <p>
 * Parse a node and return the name/value pairs
 * as two vectors:
 * </p>
 * <ol>
 *   <li>nVec - vector of keys.
 *   <li>vVec - vector of values
 * </ol>
 * @param  NodeList nl
 * @param  String hName
 * @return None
 * @throws None
 */
  public void buildNodes(Node node) {
    boolean found = false;

    if (null == node) {
      return;
    }
    int type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE: {
        // Go down the tree, starting with the root element
        buildNodes(((Document)node).getDocumentElement());
        break;
      }
      case Node.ELEMENT_NODE: {
        NodeList children = node.getChildNodes();
        if (null != children) {
          buildkey = ((Element)node).getTagName();
          int len = children.getLength();
          for (int i = 0; i < len; i++) {
            buildNodes(children.item(i));
          }
        }
        break;
      }
      case Node.TEXT_NODE:
      case Node.CDATA_SECTION_NODE: {
        String hName = buildkey;

        // Removed the trim from the following line to keep separation between ::
        //  -- Wade 
        // String chValue = ((Text)node).getData().trim(); 
        String chValue = ((Text)node).getData();
        if(nVec.isEmpty() == true){
          nVec.addElement(hName);
          vVec.addElement(chValue);
          found = false;
        } else if(nVec.isEmpty() != true){
          for(int l = 0; l < nVec.size(); l++){
            if (hName.equals(nVec.get(l))){
              found = true;
              value = (String)vVec.get(l);
              value = value + "::" + chValue;
              vVec.setElementAt(value, l);
              break;
            }
          }
          if(found == false) {
            nVec.addElement(hName);
            vVec.addElement(chValue);
            found = false;
          }
        }
        break;
      }
    }
  }
/**
 * Method: keyVector
 * <p>
 * Generate a vector of the list of tags.  
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  void keyVector(){
    if(userKey.size() != 0) {
      return;
    }

    userKey.add("userID");
    userKey.add("primaryArea");
    userKey.add("projectNumber");
    userKey.add("email");
    userKey.add("organization");
    userKey.add("title");
    userKey.add("lastName");
    userKey.add("firstName");
    userKey.add("middleName");
    userKey.add("street1");
    userKey.add("street2");
    userKey.add("mailStop");
    userKey.add("building");
    userKey.add("city");
    userKey.add("state");
    userKey.add("zipcode");
    userKey.add("zip4");
    userKey.add("officePhone");
    userKey.add("faxNumber");
    userKey.add("machineID");
    userKey.add("homeDir");
    userKey.add("queues");
    userKey.add("UID");
    userKey.add("group");
    userKey.add("codeName");
  }
/**
 * Method: getAlltags
 * <p>
 * Gett a Vector of nodes that have a specific node name.
 * </p>
 * @param  Node node
 * @param  String tag
 * @return Vector result
 * @throws None
 */
  public Vector getAllTags(Node node, String tag) {
    Vector result = new Vector();

    for(Node ch = node.getFirstChild(); ch!=null; ch = ch.getNextSibling()) {
      if(ch.getNodeType() == Node.ELEMENT_NODE) {
        if(ch.getNodeName().equals(tag)) {
          result.addElement(ch);
        }
      }
    }
    return result;
  }
}
