package entry;
/**
 * @(#)parseRU.java
 * @author  Ken Flurchick <kenf@osc.edu>
 * Date:    02/27/2000
 * Purpose: Parse the registeredUser XML file
 *          using the Xerces API.
 * @version 0.75 Removed the vector, pass  back 
 *               only the hash table.
 * @version 0.01 Initial version.
 */
import java.io.*;
import java.util.*;
import java.text.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.ProcessingInstruction;
/**
 * Class: parseRU()
 * <p>
 * Methods
 * </p>
 * <ul TYPE="square">
 *   <li>doParse(String name)
 *       <br>- Parse the file named by <i>name</i>
 *   <li>makeChildrenText(Node node)
 *       <br>- Get the node and call the next method.
 *   <li>makeChildrenText1 (Node node, StringBuffer buffer)
 *       <br>- Get the text of the node.
 * </ul>
 * @param  None
 * @return None
 * @throws None
 */
public class parseRU {
  Vector nVec = new Vector(); // nVec is the key vector.
  Vector vVec = new Vector(); // vVec is the value vector.
  static String value = null;
  Hashtable tHash = new Hashtable();
  Hashtable hash = new Hashtable();
  Vector userKey = new Vector();
  Vector userValue = new Vector();
  int nUsers = 0;
  Element root, item, itemA, itemB;
  String buildkey;
  Document doc;
/**
 * Constructor: parseFile
 * <p>
 * Empty constructor.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public void parseFile(){
    // Empty Constructor
  }
/**
 * Method: doParse
 * <ul type="square">
 *   <li>0 - OK
 *   <li>1 - Missing file name
 *   <li>2 - Parse errors
 * </ul>
 * <p>
 * Parse an XML file and return the hashtable of keys
 * and values.
 * </p>
 * @param  String name 
 * @param  String dtdFile
 * @param  Hashtable hash
 * @return integer
 * @throws None
 */
  public int doParse(String name, String dtdFile, Hashtable hash){
    String key = null, value = null;
    String hName = null;
    String lup = null;
    nUsers = 0;

    if (name == null || dtdFile == null) {
      System.err.println("Missing filename.");
      return nUsers;
    }
       
    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println("doParse Errors > zero");
      return(2);
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return(2);
    }

    // Document is well-formed

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    lup = r.getAttribute("lastUpdate");
    // System.out.println("doParse lup = " + lup);
    Vector aTags = getAllTags(r, "User");
    nUsers = aTags.size();

    // Make the hash table from the name/value pairs.
    hash.put("lastUpdate", lup);
    for(int l = 0; l < nVec.size(); l++){
      hash.put(nVec.get(l), (vVec.get(l) + "::"));
    }
    //System.out.println("Hash Table = " + hash.toString());
    return nUsers;
  }
/**
 * Method: doParseF
 * <p>
 * Parse an XML file and return the hashtable of keys
 * and values for a specific user.
 * </p>
 * @param  String name
 * @param  String dtdFile
 * @param  Hashtable thash
 * @return integer
 * @throws None
 */
  public int doParseF(String name, String dtdFile, Hashtable tHash){
    nUsers = 0;
    String key = null, value = null, hName = null;

    if (name == null || dtdFile == null) {
      System.err.println("Missing filename.");
      return nUsers;
    }

    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println("doParseF Errors > zero");
      return(2);
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return(2);
    }

    // Document is well-formed

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    Vector aTags = getAllTags(r, "User");
    nUsers = aTags.size();

    // Make the hash table from the name/value pairs.
    for(int l = 0; l < nVec.size(); l++){
      hash.put(nVec.get(l), (vVec.get(l) + "::"));
    }
    //System.out.println("Hash Table = " + hash.toString());

    // Make the tHash table (assembled by User ID rather
    // than by keywords) from the name/value pairs.
    // Zeroth element is the number of users.
    keyVector();
    // System.out.println("nUsers = " + nUsers);
    for(int k = 0; k < userKey.size(); k++){
      int startIndex = 0;
      for(int l = 0; l < nUsers; l++){
        if(k == 0){
          String tt = (String)hash.get(userKey.get(k));
          userValue.add(l, tt.substring(startIndex,
            tt.indexOf("::", startIndex)) + "::");
          startIndex = tt.indexOf("::", startIndex) + 2;
        } else {
          String tt = (String)hash.get(userKey.get(k));
          userValue.set(l, (String)userValue.elementAt(l)
            + tt.substring(startIndex,
            tt.indexOf("::", startIndex)) + "::");
          startIndex = tt.indexOf("::", startIndex) + 2;
        }
      }
    }
    for(int l = 0; l < nUsers; l++){
      String XX = (String)userValue.get(l);
      tHash.put(XX.substring(0, XX.indexOf("::")), XX);
    }
    //System.out.println("tHash Table = " + tHash.toString());

    return nUsers;
  }
/**
 * Method: removeUser()
 * <p>
 * Remove the specified user uidF.
 * </p>
 * @param  String name
 * @param  String dtdFile
 * @param  String uidF
 * @return boolean status
 * @throws None
 */
  public boolean removeUser(String name, String dtdFile, String uidF) {
    String hName = null;
    boolean status = false;
   
    String f1 = "MM  dd  yyyy  hh  mm";
    String f2 = "MM/dd/yyyy  hh:mm";
    SimpleDateFormat in = new SimpleDateFormat(f1);
    SimpleDateFormat out = new SimpleDateFormat(f2);
    Date curr;

    try {
      Calendar cal = Calendar.getInstance();
      String datestring = (
        cal.get(Calendar.MONTH) + 1) + "  "
          + cal.get(Calendar.DAY_OF_MONTH) + "  "
          + cal.get(Calendar.YEAR) + "  "
          + cal.get(Calendar.HOUR) + "  "
          + cal.get(Calendar.MINUTE);
      curr = in.parse(datestring);
    } catch (ParseException pe) {
      System.out.println("Error parsing date string");
      return false;
    }

    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println("removeUser Errors > zero");
      return false;
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return false;
    }

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    r.setAttribute("lastUpdate", out.format(curr));

    Vector aTags = getAllTags(r, "User");
    int m = aTags.size();
    for (int j = 0; j < m; j++) {
      Node a = (Node)aTags.elementAt(j);
      if( ((Element)a).getAttribute("UID").equals(uidF)) {
        r.removeChild(a);
        status = true;
      }
    }

    // Save the XML document
    try {
      PrintWriter ruFileOut = new PrintWriter(new FileWriter(name, false));
      OutputFormat format = new OutputFormat(doc, "UTF-8", true);
      XMLSerializer xmls = new XMLSerializer(ruFileOut, format);
      xmls.serialize(doc);
    } catch(IOException e){
      System.out.println("Error opening ruFileOut.");
    }
    return status;
  }
/**
 * Method: addUser
 * <p>
 * Add the new User node.
 * </p>
 * @param  String name
 * @param  String dtdFile
 * @param  Hastable uData
 * @return boolean status
 * @throws None
 */
  public String addUser(String name, String dtdFile, Hashtable uData) {
    String hName = null;
    boolean status = false;
   
    String f1 = "MM  dd  yyyy  hh  mm";
    String f2 = "MM/dd/yyyy  hh:mm";
    SimpleDateFormat in = new SimpleDateFormat(f1);
    SimpleDateFormat out = new SimpleDateFormat(f2);
    Date curr;

    try {
      Calendar cal = Calendar.getInstance();
      String datestring = (
        cal.get(Calendar.MONTH) + 1) + "  "
          + cal.get(Calendar.DAY_OF_MONTH) + "  "
          + cal.get(Calendar.YEAR) + "  "
          + cal.get(Calendar.HOUR) + "  "
          + cal.get(Calendar.MINUTE);
      curr = in.parse(datestring);
    } catch (ParseException pe) {
      System.out.println("Error parsing date string");
      status = false;
      return "Error doing date";
    }

    // Open XML file and start parsing
    DOMParser parser = new DOMParser();
    try {
      parser.parse(name);
    } catch (SAXException se) {
      System.out.println(name + " addUser Errors > zero");
      status = false;
      return "parse SAXexception";
    } catch (IOException ioe) {
      ioe.printStackTrace();
      status = false;
      return "IO Exception";
    }
    // Document is well-formed

    doc = parser.getDocument();
    removeWhiteSpaceNodes(doc);
    buildNodes(doc);
    Element r = doc.getDocumentElement();
    r.setAttribute("lastUpdate", out.format(curr));

    // Make <User> element and add it
    // as a child of root
    item = doc.createElement("User");
    item.setAttribute("UID",
      (String)uData.get("userID"));

    // Make <userID> element and add it
    // as a child of User
    itemA = doc.createElement("userID");
    itemA.appendChild(doc.createTextNode(
      (String)uData.get("userID")));
    item.appendChild(itemA);

    // Make <primaryArea> element and add it
    // as a child of User
    itemA = doc.createElement("primaryArea");
    itemA.appendChild(doc.createTextNode(
      (String)uData.get("primaryArea")));
    item.appendChild(itemA);

    // Make <projectNumber> element and add it
    // as a child of User
    itemA = doc.createElement("projectNumber");
    itemA.appendChild(doc.createTextNode(
      (String)uData.get("projectNumber")));
    item.appendChild(itemA);

    // Make <email> element and add it
    // as a child of User
    itemA = doc.createElement("email");
    itemA.appendChild(doc.createTextNode(
      (String)uData.get("email")));
    item.appendChild(itemA);

    // Make <organization> element and add it
    // as a child of User
    itemA = doc.createElement("organization");
    itemA.appendChild(doc.createTextNode(
      (String)uData.get("organization")));
    item.appendChild(itemA);

    // Make <Person> element and add it
    // as a child of User
    itemA = doc.createElement("Person");

    // Make <title> element and add it
    // as a child of Person
    itemB = doc.createElement("title");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("title")));
    itemA.appendChild(itemB);

    // Make <lastName> element and add it
    // as a child of Person
    itemB = doc.createElement("lastName");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("lastName")));
    itemA.appendChild(itemB);

    // Make <firstName> element and add it
    // as a child of Person
    itemB = doc.createElement("firstName");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("firstName")));
    itemA.appendChild(itemB);

    // Make <middleName> element and add it
    // as a child of Person
    itemB = doc.createElement("middleName");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("middleName")));
    itemA.appendChild(itemB);

    // Add <Person> as a child of <user>
    item.appendChild(itemA);

    // Make <Address> element and add it
    // as a child of User
    itemA = doc.createElement("Address");

    // Make <street1> element and add it
    // as a child of Address
    itemB = doc.createElement("street1");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("street1")));
    itemA.appendChild(itemB);

    // Make <street2> element and add it
    // as a child of Address
    itemB = doc.createElement("street2");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("street2")));
    itemA.appendChild(itemB);

    // Make <mailStop> element and add it
    // as a child of Address
    itemB = doc.createElement("mailStop");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("mailStop")));
    itemA.appendChild(itemB);

    // Make <building> element and add it
    // as a child of Address
    itemB = doc.createElement("building");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("building")));
    itemA.appendChild(itemB);

    // Make <city> element and add it
    // as a child of Address
    itemB = doc.createElement("city");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("city")));
    itemA.appendChild(itemB);

    // Make <state> element and add it
    // as a child of Address
    itemB = doc.createElement("state");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("state")));
    itemA.appendChild(itemB);

    // Make <zipcode> element and add it
    // as a child of Address
    itemB = doc.createElement("zipcode");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("zipcode")));
    itemA.appendChild(itemB);

    // Make <zip4> element and add it
    // as a child of Address
    itemB = doc.createElement("zip4");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("zip4")));
    itemA.appendChild(itemB);

    // Add <Address> as a child of <user>
    item.appendChild(itemA);

    // Make <contact> element and add it
    // as a child of User
    itemA = doc.createElement("contact");

    // Make <officePhone> element and add it
    // as a child of contact
    itemB = doc.createElement("officePhone");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("officePhone")));
    itemA.appendChild(itemB);

    // Make <faxNumber> element and add it
    // as a child of contact
    itemB = doc.createElement("faxNumber");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("faxNumber")));
    itemA.appendChild(itemB);

    // Add <contact> as a child of <user>
    item.appendChild(itemA);

    // Make <machine> element and add it
    // as a child of user
    itemA = doc.createElement("Machine");

    itemB = doc.createElement("machineID");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("machineID")));
    itemA.appendChild(itemB);

    itemB = doc.createElement("homeDir");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("homeDir")));
    itemA.appendChild(itemB);

    itemB = doc.createElement("queues");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("queues")));
    itemA.appendChild(itemB);

    itemB = doc.createElement("UID");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("UID")));
    itemA.appendChild(itemB);

    itemB = doc.createElement("group");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("group")));
    itemA.appendChild(itemB);

    item.appendChild(itemA);

    // Make <code> element and add it
    // as a child of user
    itemA = doc.createElement("code");

    itemB = doc.createElement("codeName");
    itemB.appendChild(doc.createTextNode(
      (String)uData.get("codeName")));
    itemA.appendChild(itemB);

    item.appendChild(itemA);

    r.insertBefore((Node)item, null);

    // Save the XML document
    try {
      PrintWriter ruFileOut = new PrintWriter(
        new FileWriter(name, false));
      OutputFormat format = new OutputFormat(doc, "UTF-8", true);
      XMLSerializer xmls = new XMLSerializer(ruFileOut, format);
      xmls.serialize(doc);
      status = true;
    } catch(IOException e){
      System.out.println("Error opening ruFileOut.");
    }
    return "Wrote doc OK";
  }
/**
 * Method: createProfile()
 * <p>
 * Create the user profile from the registeredUser.xml.
 * </p>
 * @param  String name
 * @param  String dtdFile
 * @param  Hastable uData
 * @return boolean status
 * @throws None
 */
  public boolean createProfile(String name, String dtdFile, Hashtable uData) {
    boolean status = false;
/*
 * Now, begin creating the profile.xml file.
 */
    try {
      Document doc = new DocumentImpl();

      String f1 = "MM  dd  yyyy  hh  mm";
      String f2 = "MM/dd/yyyy  hh:mm";
      SimpleDateFormat in = new SimpleDateFormat(f1);
      SimpleDateFormat out = new SimpleDateFormat(f2);

      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
        + cal.get(Calendar.DAY_OF_MONTH) + "  "
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + "  "
        + cal.get(Calendar.MINUTE);
      Date curr = in.parse(datestring);

      // Make <registeredUser> element root and add it
      // to the document.
      root = doc.createElement("registeredUser");
      root.setAttribute("lastUpdate", out.format(curr));

      // Make <User> element and add it
      // as a child of root
      item = doc.createElement("User");
      item.setAttribute("UID", (String)uData.get("userID"));

      // Make <primaryArea> element and add it
      // as a child of User
      itemA = doc.createElement("primaryArea");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("primaryArea")));
      item.appendChild(itemA);

      // Make <projectNumber> element and add it
      // as a child of User
      itemA = doc.createElement("projectNumber");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("projectNumber")));
      item.appendChild(itemA);

      // Make <email> element and add it
      // as a child of User
      itemA = doc.createElement("email");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("email")));
      item.appendChild(itemA);

      // Make <organization> element and add it
      // as a child of User
      itemA = doc.createElement("organization");
      itemA.appendChild(doc.createTextNode(
        (String)uData.get("organization")));
      item.appendChild(itemA);

      // Make <Person> element and add it
      // as a child of User
      itemA = doc.createElement("Person");

      // Make <title> element and add it
      // as a child of Person
      itemB = doc.createElement("title");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("title")));
      itemA.appendChild(itemB);
  
      // Make <lastName> element and add it
      // as a child of Person
      itemB = doc.createElement("lastName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("lastName")));
      itemA.appendChild(itemB);

      // Make <firstName> element and add it
      // as a child of Person
      itemB = doc.createElement("firstName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("firstName")));
      itemA.appendChild(itemB);

      // Make <middleName> element and add it
      // as a child of Person
      itemB = doc.createElement("middleName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("middleName")));
      itemA.appendChild(itemB);

      // Add <Person> as a child of <user>
      item.appendChild(itemA);

      // Make <Address> element and add it
      // as a child of User
      itemA = doc.createElement("Address");
  
      // Make <street1> element and add it
      // as a child of Address
      itemB = doc.createElement("street1");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("street1")));
      itemA.appendChild(itemB);

      // Make <street2> element and add it
      // as a child of Address
      itemB = doc.createElement("street2");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("street2")));
      itemA.appendChild(itemB);

      // Make <mailStop> element and add it
      // as a child of Address
      itemB = doc.createElement("mailStop");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("mailStop")));
      itemA.appendChild(itemB);

      // Make <building> element and add it
      // as a child of Address
      itemB = doc.createElement("building");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("building")));
      itemA.appendChild(itemB);

      // Make <city> element and add it
      // as a child of Address
      itemB = doc.createElement("city");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("city")));
      itemA.appendChild(itemB);

      // Make <state> element and add it
      // as a child of Address
      itemB = doc.createElement("state");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("state")));
      itemA.appendChild(itemB);

      // Make <zipcode> element and add it
      // as a child of Address
      itemB = doc.createElement("zipcode");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("zipcode")));
      itemA.appendChild(itemB);

      // Make <zip4> element and add it
      // as a child of Address
      itemB = doc.createElement("zip4");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("zip4")));
      itemA.appendChild(itemB);

      // Add <Address> as a child of <user>
      item.appendChild(itemA);

      // Make <contact> element and add it
      // as a child of User
      itemA = doc.createElement("contact");
  
      // Make <officePhone> element and add it
      // as a child of contact
      itemB = doc.createElement("officePhone");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("officePhone")));
      itemA.appendChild(itemB);

      // Make <faxNumber> element and add it
      // as a child of contact
      itemB = doc.createElement("faxNumber");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("faxNumber")));
      itemA.appendChild(itemB);

      // Add <contact> as a child of <user>
      item.appendChild(itemA);

      // Make <machine> element and add it
      // as a child of user
      itemA = doc.createElement("Machine");

      itemB = doc.createElement("machineID");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("machineID")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("homeDir");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("homeDir")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("queues");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("queues")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("UID");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("UID")));
      itemA.appendChild(itemB);

      itemB = doc.createElement("group");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("group")));
      itemA.appendChild(itemB);
  
      item.appendChild(itemA);
  
      // Make <code> element and add it
      // as a child of user
      itemA = doc.createElement("code");
  
      itemB = doc.createElement("codeName");
      itemB.appendChild(doc.createTextNode(
        (String)uData.get("codeName")));
      itemA.appendChild(itemB);
  
      item.appendChild(itemA);

      // Add User to the root element
      root.appendChild(item);

      // Now root is added to the doc.
      doc.appendChild(root);

      System.out.println("name = " + name);
      // Save the XML document
      try {
        PrintWriter FileOut = new PrintWriter(
          new FileWriter(name, false));
        OutputFormat format = new OutputFormat(doc, "UTF-8", true);
        XMLSerializer xmls = new XMLSerializer(FileOut, format);
        xmls.serialize(doc);
        status = true;
      } catch(IOException e){
        status = false;
        System.out.println("Error opening FileOut.");
      }
    } catch (Exception e) {
      status = false;
      e.printStackTrace();
    }
    return status;
  }
/**
 * Method: removeWhiteSpaceNodes
 * <p>
 * Remove the white space associated with a node.
 * </p>
 * @param  Element parent
 * @return None
 * @throws None
 */
  private void removeWhiteSpaceNodes(Node parent) {

    Node nextNode = parent.getFirstChild();

    // System.out.println(parent.getNodeName() + " " + 
    // parent.getChildNodes().getLength() + " " + parent.getNodeValue());

    // Deal with element nodes with no children 
      for (Node child = parent.getFirstChild(); nextNode != null;){
        child = nextNode;
        nextNode = child.getNextSibling();
        int type = child.getNodeType();

        if (Node.TEXT_NODE != type){
          removeWhiteSpaceNodes(child);
        } else if (child.getNodeValue().trim().length() == 0) {
          parent.removeChild(child);
        }
      }

      // If an element doesn't have a child at this point, it needs one
      if(parent.getNodeType() == Node.ELEMENT_NODE &&
        parent.getChildNodes().getLength() == 0) {
        // Add a dummy text node
        Text tn = doc.createTextNode("");
        parent.appendChild(tn);
      }
  }
/**
 * Method: buildNodes
 * <p>
 * Parse a node and return the name/value pairs
 * as two vectors:
 * </p>
 * <ol>
 *   <li>nVec - vector of keys.
 *   <li>vVec - vector of values
 * </ol>
 * @param  NodeList nl
 * @param  String hName
 * @return None
 * @throws None
 */
  private void buildNodes(Node node) {
    boolean found = false;

    if (null == node) {
      return;
    }
    int type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE: {
        // Go down the tree, starting with the root element
        buildNodes(((Document)node).getDocumentElement());
        break;
      }
      case Node.ELEMENT_NODE: {
        NodeList children = node.getChildNodes();
        if (null != children) {
          buildkey = ((Element)node).getTagName();
          int len = children.getLength();
          for (int i = 0; i < len; i++) {
            buildNodes(children.item(i));
          }
        }
        break;
      }
      case Node.TEXT_NODE:
      case Node.CDATA_SECTION_NODE: {
        String hName = buildkey;
        String chValue = ((Text)node).getData().trim();
        if(nVec.isEmpty() == true){
          nVec.addElement(hName);
          vVec.addElement(chValue);
          found = false;
        } else if(nVec.isEmpty() != true){
          for(int l = 0; l < nVec.size(); l++){
            if (hName.equals(nVec.get(l))){
              found = true;
              value = (String)vVec.get(l);
              value = value + "::" + chValue;
              vVec.setElementAt(value, l);
              break;
            }
          }
          if(found == false) {
            nVec.addElement(hName);
            vVec.addElement(chValue);
            found = false;
          }
        }
        break;
      }
    }
  }
/**
 * Method: keyVector
 * <p>
 * Generate a vector of the list of tags.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  void keyVector(){
    userKey.add("userID");
    userKey.add("primaryArea");
    userKey.add("projectNumber");
    userKey.add("email");
    userKey.add("organization");
    userKey.add("title");
    userKey.add("lastName");
    userKey.add("firstName");
    userKey.add("middleName");
    userKey.add("street1");
    userKey.add("street2");
    userKey.add("mailStop");
    userKey.add("building");
    userKey.add("city");
    userKey.add("state");
    userKey.add("zipcode");
    userKey.add("zip4");
    userKey.add("officePhone");
    userKey.add("faxNumber");
    userKey.add("machineID");
    userKey.add("homeDir");
    userKey.add("queues");
    userKey.add("UID");
    userKey.add("group");
    userKey.add("codeName");
  }
/**
 * Method: getAlltags
 * <p>
 * Gett a Vector of nodes that have a specific node name.
 * </p>
 * @param  Node node
 * @param  String tag
 * @return Vector result
 * @throws None
 */
  private Vector getAllTags(Node node, String tag) {
    Vector result = new Vector();

    for(Node ch = node.getFirstChild(); ch!=null; ch = ch.getNextSibling()) {
      if(ch.getNodeType() == Node.ELEMENT_NODE) {
        if(ch.getNodeName().equals(tag)) {
          result.addElement(ch);
        }
      }
    }
    return result;
  }
}
