package test.pdt;
/**
 * @(#) TESTpdtBean.java
 * @author  Ken Flurchick
 * Date:    07/18/2000
 * Purpose: Bean for the adder application page of 
 *          the Problem Description Test Toolkit.
 *
 * Required Classes:
 *          WebFlow Package
 */
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import WebFlow.*;
import WebFlow.event.*;
import WebFlow.ContextManager.*;
import org.omg.CORBA.*;
import java.util.*;
import java.text.*;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.*;
/**
 * Class: TESTpdtBean
 * <p>
 * This class will process the TEST PDT form.
 * </p>
 * <pre>
 * The TESTpdtBean process is
 *              P1.html 
 *               ||
 *               ||
 *               \/
 *        P1.jsp <=> TESTpdtBean
 * </pre>
 * @param  None
 * @return None
 * @throws None
 */
public class TESTpdtBean{
  ServletOutputStream out;
  ContextManager cm;
  String title; 
  String desc;
  String documentRoot;
  String user;
  String problemDescriptionName;
  Element item;
/**
 * Method: TESTpdtBean
 * <p>
 * Empty constructor.
 * </p>
 * @param  None
 * @return None
 * @throws None
 */
  public TESTpdtBean(){
    // Empty constructor.
  }
// Setter methods for the Bean.
  public void setTitle(String S){
    this.title = S;
  }
  public void setDesc(String S){
    this.desc = S;
  }
  public void setDocumentRoot(String S) {
    this.documentRoot = S;
  }
  public void setUser(String S){
    this.user = S;
  }
  public void setProblemDescriptionName(String S){
    this.problemDescriptionName = S;
  }
// Getter methods for the Bean.
  public String getTitle(){
    return title;
  }
  public String getDesc(){
    return desc;
  }
  public String getDocumentRoot(){
    return documentRoot;
  }
  public String getUser(){
    return user;
  }
  public String getProblemDescriptionName(){
    return problemDescriptionName;
  }
/**
 * Method: updatePDfile
 * <p>
 * Update the <em>PD.xml</em> file with the 
 * domain specific information.
 * </p>
 * @param  None
 * @return boolean status
 * @throws None
 */
  public boolean updatePDfile() {
    String pdFile;
    boolean status = false;
    String sStatus;

    pdFile = documentRoot + "WEB-INF/Descriptors/users/" 
      + user + File.separator + problemDescriptionName 
      + File.separator + problemDescriptionName + ".xml";
    System.out.println("pdFile = " + pdFile);

    try {
      String f1 = "MM  dd  yyyy  hh  mm";
      String f2 = "MM/dd/yyyy  hh:mm";
      SimpleDateFormat in = new SimpleDateFormat(f1);
      SimpleDateFormat out = new SimpleDateFormat(f2);

      Calendar cal = Calendar.getInstance();
      String datestring = (cal.get(Calendar.MONTH) + 1) + "  "
        + cal.get(Calendar.DAY_OF_MONTH) + "  "
        + cal.get(Calendar.YEAR) + "  "
        + cal.get(Calendar.HOUR) + "  "
        + cal.get(Calendar.MINUTE);
      Date curr = in.parse(datestring);
      long currentMillis = System.currentTimeMillis();

      // Start parsing
      DOMParser parser = new DOMParser();
      parser.parse(pdFile);

      // Update the root tag time stamp.
      Document doc = parser.getDocument();
      Element root = doc.getDocumentElement();
      root.setAttribute("timeStamp", Long.toString(currentMillis));

      // Make <applications> element and add it
      // as a child of problem
      item = doc.createElement("applications");
      item.setAttribute("version", "1.0");
      item.appendChild(doc.createTextNode("Adder "));
      root.appendChild(item);

      // Make <title> element and add it
      // as a child of problem
      item = doc.createElement("title");
      item.appendChild(doc.createTextNode(title));
      root.appendChild(item);

      // Make <description> element and add it
      // as a child of problem
      item = doc.createElement("description");
      item.appendChild(doc.createTextNode(desc));
      root.appendChild(item);

      // Now root is inserted into the doc.
      // root.insertBefore(item, null);

      try{
        PrintWriter fileOut = new PrintWriter(
          new FileWriter(pdFile, false));
        OutputFormat format = new OutputFormat(doc, null, true);
        format.setMethod("XML");
        format.setIndent(2);
        XMLSerializer serializer = new XMLSerializer(fileOut, format);
        serializer.serialize(doc);
        sStatus = "Problem Description File for " + problemDescriptionName
          + "successfully created.";
        status = true;
      } // End of try block for fileOut.
      catch(IOException e)
      {
        System.out.println("Error opening fileOut.");
      }
    } catch (Exception e) {
      sStatus = "Error in docImpl.";
      status = false;
      e.printStackTrace();
    }
    return status;
  }
/**
 * Method: getBeanInfo
 * @param  None
 * @return None
 * @throws None
 */
  public String getBeanInfo(){
    return "By K. Flurchick and T. Haupt";
  }
}
