import java.io.*;
import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class EchoXML extends DefaultHandler
{
  public EchoXML ()
  {
    super();
  }

  /** Processing instruction. */
  public void processingInstruction(String target, String data) 
  {
    System.out.print("<?");
    System.out.print(target);
    if (data != null && data.length() > 0)
    {
      System.out.print(' ');
      System.out.print(data);
    }
    System.out.print("?>");

  } // processingInstruction(String,String)

  /** Start document. */
  public void startDocument() 
  {
    System.out.println("<?xml version=\"1.0\"?>");
  } // startDocument()

  /** Start element. */
  public void startElement(String uri, String name, 
	String qName, Attributes atts)
  {
    System.out.print("<");
    System.out.print(qName);
    for (int i = 0; i < atts.getLength(); i++)
    {
      System.out.print(" ");
      System.out.print(atts.getQName(i));
      System.out.print("=\"");
      System.out.print(atts.getValue(i));
      System.out.print("\"");
    }
    System.out.print(">");
    
  } // startElement(String,AttributeList)

  /** Characters. */
  public void characters(char ch[], int start, int length) 
  {
    System.out.print(new String(ch, start, length));
  } // characters(char[],int,int);

  /** Ignorable whitespace. */
  public void ignorableWhitespace(char ch[], int start, int length) 
  {
    characters(ch, start, length);
  } // ignorableWhitespace(char[],int,int);

  /** End element. */
  public void endElement(String uri, String name, String qName) 
  {
    System.out.print("</");
    System.out.print(qName);
    System.out.print(">");
  } // endElement(String)

  /** End document. */
  public void endDocument() 
  {
    System.out.println("End of Document");
    // No need to do anything.
  } // endDocument()

  //
  // ErrorHandler methods
  //

  /** Warning. */
  public void warning(SAXParseException ex) 
  {
    System.err.println("[Warning] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  /** Error. */
  public void error(SAXParseException ex) 
  {
    System.err.println("[Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  /** Fatal error. */
  public void fatalError(SAXParseException ex) 
    throws SAXException 
  {
    System.err.println("[Fatal Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
    throw ex;
  }

  /** Returns a string of the location. */
  private String getLocationString(SAXParseException ex) 
  {
    StringBuffer str = new StringBuffer();

    String systemId = ex.getSystemId();
    if (systemId != null)
    {
      int index = systemId.lastIndexOf('/');
      if (index != -1)
        systemId = systemId.substring(index + 1);
      str.append(systemId);
    }
    str.append(':');
    str.append(ex.getLineNumber());
    str.append(':');
    str.append(ex.getColumnNumber());

    return str.toString();
  } // getLocationString(SAXParseException):String

  /** Main program entry point. */
  public static void main(String argv[]) throws Exception
  {
    if (argv.length == 0)
    {
      System.out.println("Usage:  java EchoXML uri");
      System.out.println("   where uri is the URI of your XML document.");
      System.out.println("   Sample:  java EchoXML sonnet.xml");
      System.exit(1);
    }

    XMLReader xr = new SAXParser();
    EchoXML handler = new EchoXML();
    xr.setContentHandler(handler);
    xr.setErrorHandler(handler);
    xr.setFeature("http://xml.org/sax/features/validation", true);
    xr.setFeature("http://xml.org/sax/features/namespaces", false);
    xr.setFeature("http://xml.org/sax/features/namespace-prefixes", false);
    for (int i = 0; i < argv.length; i++)
    {
      xr.parse(new InputSource(argv[i]));
    }
  } // main(String[])
} 
