import java.lang.reflect.Array;

import org.xml.sax.XMLReader;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.apache.xerces.dom.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import java.io.*;
import java.util.*;

/**
 * A JavaBean which uses SAX2 for extracting data
 * from an XML file. Validation is performed.
 *
 * @author Armen B. Ezekielian
 * @version 1.0
 */
public class TestOutput2
{
  /** 
   * The URI of the xml file to be processed. Must be set prior
   * to its use in methods typically by a request parameter or
   * by a bean <code>&lt;jsp:setProperty&gt;</code> bean tag
   * in a JSP page.
   */
  private String XMLFileName; 

  private String DTDFileName;

  /**
   * The name of the element to use in the processing of 
   * the xml file. Must be set prior to its use in methods
   * typically by a request parameter or by a bean 
   * <code>&lt;jsp:setProperty&gt;</code> bean tag
   * in a JSP page.
   */
  private String elementName; 

  /** 
   * The name of the attribute to use in the processing
   * of the xml file. Must be set prior to its use in methods
   * typically by a request parameter or by a bean 
   * <code>&lt;jsp:setProperty&gt;</code> bean tag
   * in a JSP page.
   */
  private String attName; 

  /** 
   * The value of the attribute to use in the processing
   * of the xml file. Must be set prior to its use in methods
   * typically by a request parameter or by a bean 
   * <code>&lt;jsp:setProperty&gt;</code> bean tag
   * in a JSP page.
   */
  private String attValue; 

  /**
   * Holds all the <em>element data</em>
   * (<em>parsed character data</em>) of the XML file
   */
  private String data;

  /**
   * Flag to indicate whether the parser reported any errors.
   */
  private boolean isError;

  /**
   * Contains descriptive message destined for client browser.
   */
  private String errorMsg;

  /**
   * Contains descriptive message destined for client browser.
   */
  private String fatalErrorMsg;

  /**
   * Requisite no-argument constructor. Performs basic default
   * initialization of instance variables.
   */
  public TestOutput2()
  {
    super();

    // Not really necessary to initialize these since instance
    // variables get sensible defaults from Java. However, any
    // method in this class which makes use of any of the 
    // the four following fields must have these fields set
    // to valid values prior to its use. So we note these 
    // special values here. The methods do perform checks for 
    // valid values.
    XMLFileName = ""; // used by all getXXX methods
    elementName = ""; // used by all getXXX methods
    attName = ""; // used by some getXXX methods
    attValue = ""; // used by some getXXX methods

    // Initialize error flag
    isError = false;

  } // SAXUtilsBean():constructor

  /**
   * Returns the <code>(#PCDATA)</code> from all XML elements
   * with the given {@link elementName}
   *
   * @return A <code>String</code> array containing the data 
   * @exception java.io.IOException
   *            If the XML file or DTD cannot be found or opened
   * @exception org.xml.sax.SAXException
   *            Any SAX exception, possibly wrapping another 
   *            exception
   */
  public setData(String ) throws SAXParseException
  {

    boolean isEmpty = false;
    File XMLFile = new File(XMLFileName);
    if(XMLFile != null)
    {
      XMLSerializer serializer = new XMLSerializer();
      try
      {
        serializer.setOutputCharStream(new FileWriter(XMLFile));
      }
      catch(IOException ioExcp)
      {
        System.out.println("Save failed: " + ioExcp.toString() );
      }    
    }

    // Perform the parsing operations
    XMLReader xr = new SAXParser();
    xr.setContentHandler(new XMLSerializer());
    xr.setErrorHandler(new ErrHandler());
    try
    {
      xr.setFeature("http://xml.org/sax/features/validation", true);
      xr.setFeature("http://xml.org/sax/features/namespaces", false);
      xr.setFeature("http://xml.org/sax/features/namespace-prefixes", false);
      xr.parse(new InputSource(XMLFileName));
    }
    catch(SAXNotSupportedException snse)
    {
      Exception ex = snse;
      if(snse.getException() != null)      
        ex = snse.getException();
      throw new SAXParseException(ex.getMessage(), locator, ex);
    }
    catch(SAXNotRecognizedException snre)
    {
      Exception ex = snre;
      if(snre.getException() != null)      
        ex = snre.getException();
      throw new SAXParseException(ex.getMessage(), locator, ex);
    }
    catch(SAXParseException spe)
    {
      Exception ex = spe;
      if(spe.getException() != null)      
        ex = spe.getException();
      String msg = ex.getMessage() + "\n" +
                   "SAXParseException caught in " + locator.getSystemId() +
                   " at line " + locator.getLineNumber() +
                   " column " + locator.getColumnNumber();
      throw new SAXParseException(msg, locator, ex);
    }
    catch(SAXException se)
    {
      Exception ex = se;
      if(se.getException() != null)      
        ex = se.getException();
      String msg = ex.getMessage() + "\n" +
                   "SAXException caught in " + locator.getSystemId() +
                   " at line " + locator.getLineNumber() +
                   " column " + locator.getColumnNumber();
      throw new SAXParseException(msg, locator, ex);
    }
    /*
    catch(NullPointerException npe)
    {
      throw new SAXParseException(npe.getMessage(), locator, npe);
    }
    */
    catch(IOException ioe)
    {
      throw new SAXParseException(ioe.getMessage(), locator, ioe);
    }

    // Check if any element is non-empty. If all elements are
    // empty, including any amount of pure whitespace, return 
    // the NO_DATA constant.
    data = ssa.toArray();
    int numData = Array.getLength(data);
    if(numData == 0) isEmpty = true;
    
    if(isEmpty)
    { 
      data = new String[] { NO_DATA };
      return data;
    } else return data;
      
  } // getData():String[]

  /**
   * Returns the <code>(#PCDATA)</code> from all XML elements
   * having a given {@link attName}
   *
   * @return A <code>String</code> array containing the data
   * @exception java.io.IOException
   *            If the XML file or DTD cannot be found or opened
   * @exception org.xml.sax.SAXException
   *            Any SAX exception, possibly wrapping another
   *            exception
   */
  public String[] getDataByAttribute() throws Exception
  {
    return new String[] { "0", "1" };
  } // getDataByAttribute():String[]

  /**
   * Returns the <code>(#PCDATA)</code> from all XML elements
   * having a given {@link attName}-{@link attValue} pair
   *
   * @return A <code>String</code> array containing the data
   * @exception java.io.IOException
   *            If the XML file or DTD cannot be found or opened
   * @exception org.xml.sax.SAXException
   *            Any SAX exception, possibly wrapping another
   *            exception
   */
  public String[] getDataByAttValPair() throws Exception
  {
    return new String[] { "0", "1" };
  } // getDataByAttValPair():String[]

  /**
   * Returns the {@link attValue} from all XML elements
   * having a given {@link elementName}
   *
   * @return A <code>String</code> array containing the data
   * @exception java.io.IOException
   *            If the XML file or DTD cannot be found or opened
   * @exception org.xml.sax.SAXException
   *            Any SAX exception, possibly wrapping another
   *            exception
   */
  public String[] getAttValsByElement() throws Exception
  {
    return new String[] { "0", "1" };
  } // getAttValsByElement():String[]

  public boolean getIsError()
  {
    return isError;
  }

  public String getErrorMessages()
  {
    String errorMessages = errorMsg + "\n" + fatalErrorMsg; 
    return errorMessages;
  }

  public void setXMLFileName(String name)
  {
    XMLFileName = name;
  }

  public void setDTDFileName(String name)
  {
    DTDFileName = name;
  }

  public void setElementName(String name)
  {
    elementName = name;
  }

  class ErrHandler extends DefaultHandler
  {
    ErrHandler()
    { 
      super();
    }

    /* Warning */
    public void warning(SAXParseException spe) throws SAXParseException
    {
      System.err.println("[Warning] "+
                         getLocationString(spe)+": "+
                         spe.getMessage());
      throw spe;
    }

    /* Error */
    public void error(SAXParseException spe) throws SAXParseException
    {
      String errMsg = "[Error] "+getLocationString(spe)+": "+
                      spe.getMessage();
      System.err.println(errMsg);
      isError = true;
      errorMsg = errMsg;
      throw spe;
    }

    /* Fatal error */
    public void fatalError(SAXParseException spe)
      throws SAXParseException
    {
      String errMsg = "[Fatal Error] "+getLocationString(spe)+": "+
                      spe.getMessage();
      System.err.println(errMsg);
      isError = true;
      fatalErrorMsg = errMsg;
      throw spe;
    }

    /** Returns a string of the location. */
    private String getLocationString(SAXParseException spe)
    {
      StringBuffer str = new StringBuffer();

      String systemId = spe.getSystemId();
      if (systemId != null)
      {
        int index = systemId.lastIndexOf('/');
        if (index != -1)
          systemId = systemId.substring(index + 1);
        str.append(systemId);
      }
      str.append(':');
      str.append(spe.getLineNumber());;
      str.append(':');
      str.append(spe.getColumnNumber());

      return str.toString();
    } // getLocationString(SAXParseException):String 
  } // class ErrHandler

  private class ElementDetails
  {
    public String qName;
    public Attributes atts;
    public ElementDetails(String qName, Attributes atts)
    {
      this.qName = qName;
      this.atts = new AttributesImpl(atts); // make a copy
    }
  }
} // public class SAXUtilsBean
