/*
 * $Header: /home/cvspublic/jakarta-struts/src/share/org/apache/struts/taglib/CancelTag.java,v 1.2 2000/06/24 03:16:11 craigmcc Exp $
 * $Revision: 1.2 $
 * $Date: 2000/06/24 03:16:11 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */


package org.apache.struts.taglib;


import java.io.IOException;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;
import org.apache.struts.util.BeanUtils;
import org.apache.struts.util.MessageResources;


/**
 * Tag for input fields of type "cancel".
 *
 * @author Jeff Hutchinson
 * @version $Revision: 1.2 $ $Date: 2000/06/24 03:16:11 $
 */

public final class CancelTag extends BaseHandlerTag {


    // ----------------------------------------------------- Instance Variables

    /**
     * The message resources for this package.
     */
    private static MessageResources messages =
	MessageResources.getMessageResources
	("org.apache.struts.taglib.LocalStrings");


    /**
     * The property name of the generated button.
     */
    private String property = Constants.CANCEL_PROPERTY;


    /**
     * The value of the button label.
     */
    private String value = null;


    // ------------------------------------------------------------- Properties


    /**
     * Return the property name.
     */
    public String getProperty() {
        return (property);
    }

    /**
     * Set the property name.
     *
     * @param name The property name
     */
    public void setProperty(String property) {
        this.property = property;
    }


    /**
     * Return the label value.
     */
    public String getValue() {
        return (value);
    }


    /**
     * Set the label value.
     * @param value The label value
     */
    public void setValue(String value) {
        this.value = value;
    }

    // --------------------------------------------------------- Public Methods

    /**
     * Process the start of this tag.
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        // Do nothing until doEndTag() is called
        return (EVAL_BODY_TAG);
    }

    /**
     * Process the end of this tag.
     * @exception JspException if a JSP exception has occurred
     */
    public int doEndTag() throws JspException {

    // Acquire the label value we will be generating
    String label = value;
    if ((label == null) && (bodyContent != null))
        label = bodyContent.getString().trim();
    if ((label == null) || (label.trim().length() < 1))
        label = "Cancel";

    // Generate an HTML element
    StringBuffer results = new StringBuffer();
    results.append("<input type=\"submit\"");
    results.append(" name=\"");
    results.append(property);
    results.append("\"");
    if (accessKey != null) {
	results.append(" accesskey=\"");
	results.append(accessKey);
	results.append("\"");
    }
    if (tabIndex != null) {
	results.append(" tabindex=\"");
	results.append(tabIndex);
	results.append("\"");
    }
    results.append(" value=\"");
    results.append(label);
    results.append("\"");
    results.append(prepareEventHandlers());
    results.append(prepareStyles());
    results.append(">");

    // Render this element to our writer
    JspWriter writer = pageContext.getOut();
    try {
        writer.print(results.toString());
    }
    catch (IOException e) {
        throw new JspException
        (messages.getMessage("baseFieldTag.io", e.toString()));
    }

    return (EVAL_PAGE);

    }


}
