/**
 * @(#)Transform
 * @author   Wade Lutgen <wade@pacific.mps.ohio-state.edu>
 * Date:  07/19/2000
 * Purpose:  Given an XML and XSL file, this program creates an HTML file 
 *
 * Required Classes: none
 */

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xalan.*;
import java.lang.reflect.*;
import org.apache.xalan.xslt.*;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.xml.*;

/**
 * Class: Transform 
 * <p>
 * This is the main class for <strong>Transform</strong>
 * </p>
 * @param   None
 * @return  None
 * @throws  None
 */
public class Transform 
{

/**
 * Constructor: Transform 
 * <p>
 *   Empty Constructor 
 * </p>
 * @param   None 
 * @return  None
 * @throws  None 
 */
  Transform() {
  }

/**
 * Constructor: Transform
 * <p>
 *   This constructor will take a xml file, a xsl file, and a html file and call writeHTML()
 * </p>
 * @param   String xmlFileName - input file.  Must be xml
 * @param   String xslFileName - input file.  Must be xsl
 * @param   String htmlFileName - output file.  This is html 
 * @return  None
 * @throws  None
 */
  Transform(String xmlFileName, String xslFileName, String htmlFileName) {

    writeHTML(xmlFileName, htmlFileName, xslFileName);
  }

/**
 * Method: writeHTML
 * <p>
 *   This member parses the inputs and writes the output file 
 * </p>
 * @param   String xmlFileName - input file.  Must be xml
 * @param   String xslFileName - input file.  Must be xsl
 * @param   String htmlFileName - output file.  This is html 
 * @return  None
 * @throws  None
 */
  public void writeHTML( String xmlFileName, String xslFileName, 
    String htmlFileName ) {

    XMLParserLiaison xmlProcessorLiaison = null;
    String parserLiaisonClassName = Constants.LIAISON_CLASS;
    try {
      Class parserLiaisonClass = Class.forName(parserLiaisonClassName);
      Constructor parserLiaisonCtor = parserLiaisonClass.getConstructor(null);
      xmlProcessorLiaison=(XMLParserLiaison)parserLiaisonCtor.newInstance(null);
    }
    catch(Exception e) {
      System.err.println("Could not create XML Processor Liaison....exiting");
      System.exit(0);
    }

    XSLTProcessor processor = XSLTProcessorFactory.getProcessor(
                              xmlProcessorLiaison);

    xmlProcessorLiaison.setIndent(2);
 //   xmlProcessorLiaison.setUseValidation(true);

    try {
      StylesheetRoot stylesheet = processor.processStylesheet(xslFileName);
      OutputStream outputStream = new FileOutputStream(htmlFileName);
      stylesheet.setOutputMethod("html");
      stylesheet.m_useXercesSerializers = true;
      Node sourceTree = processor.getSourceTreeFromInput(
                        new XSLTInputSource((xmlFileName)));
      stylesheet.process(processor, sourceTree, 
                         new XSLTResultTarget(outputStream));
    }
    catch(SAXException se) {
      System.err.println("SAXException.  Exiting");
      System.exit(0);
    }
    catch(FileNotFoundException fne) {
      System.err.println("File not found.  Exiting");
      System.exit(0);
    }
    catch(IOException ioe) {
      System.err.println("IO excepction.  Exiting.");
      System.exit(0);
    }
    catch(java.lang.NoClassDefFoundError ncdfe) {
      System.err.println("A Class was not found. Exiting");
      System.exit(0);
    }
    catch(RuntimeException rte) {
      System.err.println("Runtime error.  Exiting");
      System.exit(0);
    }
  }

  public static void main(String argv[]) {

    if(argv.length < 6) {
      System.err.println("The command must be of the form \"java Transform" +
       " -xml foo.xml -xsl foo.xsl -html foo.html\"");
      System.exit(0);
    }
  
    String xmlFile = null;
    String xslFile = null;
    String htmlFile = null;
 
    for (int i = 0; i < argv.length; i++) {
      if("-xml".equalsIgnoreCase(argv[i])) {
        xmlFile = argv[++i];
      }
      else if("-xsl".equalsIgnoreCase(argv[i])) {
        xslFile = argv[++i];
      }
      else if("-html".equalsIgnoreCase(argv[i])) {
        htmlFile = argv[++i];
      }
    }
     
    Transform myapp = new Transform();
    myapp.writeHTML(xmlFile, xslFile, htmlFile);    
  }
}
